package com.baojia.mebike.dialog

import android.content.DialogInterface
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import androidx.fragment.app.FragmentManager
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseCompatDialogFragment
import com.baojia.mebike.callback.IDialogCallback
import com.baojia.mebike.data.CommData
import com.baojia.mebike.util.DensityUtil
import kotlinx.android.synthetic.main.dialog_double_round_button.*

/**
 * +----------------------------------------------------------------------
 * | @time 2018/12/25:2:21 PM
 * | @author house
 * | @class_describe
 * +----------------------------------------------------------------------
 **/
class LeftRightButtonTipsDialog : BaseCompatDialogFragment(), View.OnClickListener {
    override fun onClick(v: View?) {
        when (v) {
            oneButtonConfirm,
            cancelButton -> {
                dialogCallback?.onCancel()
                dismiss()
            }
            confirmButton -> {
                dialogCallback?.onConfirm()
                dismiss()
            }

        }
    }

    companion object {
        @JvmStatic
        fun show(fragmentManager: FragmentManager, title: String?, content: String?, buttonCount: Int = 1, tipsIcon: Int = 0, cancelValue: String? = "", confirmValue: String? = ""): LeftRightButtonTipsDialog {
            val dialog = LeftRightButtonTipsDialog()
            val bundle = Bundle()
            bundle.putString("title", title)
            bundle.putString("content", content)
            bundle.putString("confirmValue", confirmValue)
            bundle.putString("cancelValue", cancelValue)
            bundle.putInt("buttonCount", buttonCount)
            bundle.putInt("tipsIcon", tipsIcon)
            bundle.putBoolean("cancelable", true)
            dialog.arguments = bundle
            dialog.show(fragmentManager, "LeftRightButtonTipsDialog")
            return dialog
        }

        @JvmStatic
        fun showUnCancle(fragmentManager: FragmentManager, title: String?, content: String?, buttonCount: Int = 1, tipsIcon: Int = 0, cancelValue: String? = "", confirmValue: String? = "", cancelable: Boolean): LeftRightButtonTipsDialog {
            val dialog = LeftRightButtonTipsDialog()
            val bundle = Bundle()
            bundle.putString("title", title)
            bundle.putString("content", content)
            bundle.putString("confirmValue", confirmValue)
            bundle.putString("cancelValue", cancelValue)
            bundle.putInt("buttonCount", buttonCount)
            bundle.putInt("tipsIcon", tipsIcon)
            bundle.putBoolean("cancelable", cancelable)
            dialog.arguments = bundle
            dialog.show(fragmentManager, "LeftRightButtonTipsDialog")
            return dialog
        }

    }

    private var tipsIcon: Int = 0
    private var content: String? = null
    private var title: String? = null
    private var cancelValue: String? = null
    private var confirmValue: String? = null
    private var buttonCount: Int = 2
    private var dialogCallback: IDialogCallback? = null
    private var cancelable_ = true

    fun setDialogCallback(dialogCallback: IDialogCallback) {
        this.dialogCallback = dialogCallback
    }

    override fun setContentView(): Int {
        return R.layout.dialog_double_round_button
    }


    override fun bindView() {

        tipsIcon = bundle.getInt("tipsIcon")
        title = bundle.getString("title")
        cancelValue = bundle.getString("cancelValue")
        confirmValue = bundle.getString("confirmValue")
        content = bundle.getString("content")
        buttonCount = bundle.getInt("buttonCount", 2)
        cancelable_ = bundle.getBoolean("cancelable")

        if (tipsIcon == 0) {
            tipIconImageView.visibility = View.GONE
        } else {
            tipIconImageView.visibility = View.VISIBLE
            tipIconImageView.setImageResource(tipsIcon)
        }

        if (buttonCount == 2) {
            cancelButton.visibility = View.VISIBLE
            confirmButton.visibility = View.VISIBLE
            oneButtonConfirm.visibility = View.GONE
        } else {
            cancelButton.visibility = View.GONE
            confirmButton.visibility = View.GONE
            oneButtonConfirm.visibility = View.VISIBLE
        }

        if (!TextUtils.isEmpty(cancelValue)) {
            cancelButton.text = cancelValue
        }

        if (!TextUtils.isEmpty(confirmValue)) {
            confirmButton.text = confirmValue
        }

        if (TextUtils.isEmpty(title)) {
            titleTextView.visibility = View.GONE
        } else {
            titleTextView.visibility = View.VISIBLE
            titleTextView.text = title
        }

        content?.let {
            contentTextView.text = content
        }

        oneButtonConfirm.setOnClickListener(this)
        confirmButton.setOnClickListener(this)
        cancelButton.setOnClickListener(this)
    }

    override fun getWidth(): Int {
        return CommData.getScreen()[0] - DensityUtil.dip2px(activity, 50f)
    }

    override fun cancelable(): Boolean = cancelable_

    override fun onCancel(dialog: DialogInterface?) {
        super.onCancel(dialog)
        dialogCallback?.onCancel()
    }
}