package com.baojia.mebike.feature.exclusive.iWillExchange

import android.app.Activity
import android.text.TextUtils
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.data.response.exclusive.IWillExchangeResponse
import com.baojia.mebike.data.response.exclusive.SubmitChangeBatteryResponse
import com.baojia.mebike.feature.model.PayModel
import com.baojia.mebike.http.HttpUtils
import com.baojia.mebike.util.Toast
import com.baojia.mebike.util.ToolBox
import io.reactivex.disposables.Disposable

class IwillExchangeModel(activity: Activity) : PayModel(activity) {
    private val URL_IWILL_EXCHANGE = "/bike/knight/change/list/v2"
    // 个人专属换电列表
    private val URL_IWILL_EXCHANGE_PERSONAL = "/pmall/get/exchange/list"
    // 免费换电下单
    private val SUBMIT_CHANGE_BATTERY_ORDER = "/bike/knight/generate/free/changeOrder"

    fun submitChangeBatteryOrder(bikeId: Int, subType: Int = 2, orderNo: String? = null, httpCallback: HttpCallback<SubmitChangeBatteryResponse.DataBean>?): Disposable {
        val params = mutableMapOf<String, Any>()
        params["bikeId"] = bikeId
        params["subType"] = subType
        if (!TextUtils.isEmpty(orderNo)) {
            params["orderNo"] = orderNo!!
        }
        return HttpUtils.postRequest(activity, SUBMIT_CHANGE_BATTERY_ORDER, params,
                object : HttpCallback<SubmitChangeBatteryResponse>() {
                    override fun onSucceed(data: SubmitChangeBatteryResponse?) {
                        super.onSucceed(data)
                        if (data?.data != null) {
                            httpCallback?.onSucceed(data.data)
                        }
                    }

                    override fun onFailed(code: Int, message: String?) {
                        super.onFailed(code, message)
                        Toast.show(activity, message)
                    }

                    override fun onError(code: Int, message: String?) {
                        super.onError(code, message)
                        onFailed(code, message)
                    }
                }, SubmitChangeBatteryResponse::class.java)
    }

    fun getData(bikeId: Int, orderNo: String?, httpCallback: HttpCallback<IWillExchangeResponse.DataBean>?): Disposable {

        val params = mutableMapOf<String, Any?>("bikeId" to bikeId)

        if (!TextUtils.isEmpty(orderNo)) {
            params["orderNo"] = orderNo
        }
        var url:String = if (ToolBox.isCompanyExclusive()) {
            // 企业
            URL_IWILL_EXCHANGE
        } else {
            // 个人
            URL_IWILL_EXCHANGE_PERSONAL
        }
        return HttpUtils.postRequest(activity, url, params,
                object : HttpCallback<IWillExchangeResponse>() {

                    override fun onSucceed(data: IWillExchangeResponse?) {
                        super.onSucceed(data)

                        data?.data?.let {
                            httpCallback?.onSucceed(it)
                        }

                    }

                    override fun onFailed(code: Int, message: String?) {
                        super.onFailed(code, message)
                        Toast.show(activity, message)
                    }

                    override fun onError(code: Int, message: String?) {
                        super.onError(code, message)
                        onFailed(code, message)
                    }
                }, IWillExchangeResponse::class.java)
    }

}