package com.baojia.mebike.feature.exclusive.iWillExchange

import android.app.Activity
import android.text.TextUtils
import com.baojia.mebike.base.BasePresenter
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.config.Config
import com.baojia.mebike.data.response.BaseResponse
import com.baojia.mebike.data.response.exclusive.IWillExchangeResponse
import com.baojia.mebike.data.response.exclusive.SubmitChangeBatteryResponse
import com.baojia.mebike.data.response.order.PayByOtherResponse
import com.baojia.mebike.feature.exclusive.shopping.sendprogress.SendProgressActivity
import com.baojia.mebike.util.ToolBox

class IwillExchangePresenter( activity: Activity, val mView: IwillExchangeContract.View) : BasePresenter(activity), IwillExchangeContract.Presenter {


    override fun dataBean(): IWillExchangeResponse.DataBean?  = dataBean


    private val model: IwillExchangeModel? = IwillExchangeModel(activity)

    private var orderNo: String? = null
    private var changeBatteryId: Long = 0

    private var dataBean : IWillExchangeResponse.DataBean? = null

    override fun getData() {
        addDisposable(model?.getData(mView.bikeId(), mView.orderNo(), object : HttpCallback<IWillExchangeResponse.DataBean>() {
            override fun onSucceed(data: IWillExchangeResponse.DataBean?) {
                super.onSucceed(data)

                mView?.setIntroduceData(data?.knowListTitle, data?.knowList)
                mView?.setData(data?.list)

                dataBean = data

                if (!TextUtils.isEmpty(data?.changeOrderNo)) {
                    mView.setOrderNo(data?.changeOrderNo)
                }

                if (!ToolBox.isCompanyExclusive()) {
                    if (data?.bought == 0) {
                        // 显示无套餐图片
                        mView.showBuyLayout(data?.imgUrl3)
                    } else if (data?.bought == 1) {
                        // 显示续期
                        mView.showRenewalLayout(data?.buttonText,data?.placeholder,data?.title)
                    }
                }
            }
        }))

    }

    override fun payRequest() {
        var paymentType = 15
        if (ToolBox.isCompanyExclusive()) {
            paymentType = 15
        } else {
            paymentType = 17
        }
        if (mView.paymentType() > 0) {
            paymentType = mView.paymentType()
        }
        addDisposable(model?.changePowerPrePay(mView.channelMethod(), paymentType, mView.bikeId(), mView.changeBatteryId(), mView.orderNo(), object : HttpCallback<PayByOtherResponse.DataBean>() {
            override fun onSucceed(data: PayByOtherResponse.DataBean) {
                super.onSucceed(data)

                orderNo = data.orderNo
                changeBatteryId = data.changeBatteryId

                if (data.payChannelId == Config.CODE_PAYCHANNLID_WECHAT) {
                    // 微信支付
                    mView.wechatPay(data)
                } else if (data.payChannelId == Config.CODE_PAYCHANNLID_ALIPAY) {
                    // 支付宝支付
                    mView.aliPay(data)
                }
            }
        }))
    }

    override fun checkOrderPayStatus() {
        if (TextUtils.isEmpty(orderNo)) {
            return
        }
        addDisposable(model?.checkOrderPayStatus(orderNo, 4, object : HttpCallback<BaseResponse?>() {
            override fun onSucceed(data: BaseResponse?) {
                super.onSucceed(data)
                data?.let {
                    orderNo = null
                    if (changeBatteryId != null) {
                        // 跳转换电进度
                        SendProgressActivity.start(context,1,"",changeBatteryId,2)
                    }
                }
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)
            }
        }))
    }


    override fun exchangePower() {
        addDisposable(model?.submitChangeBatteryOrder(mView.bikeId(), mView.subType(), mView.orderNo(),
                object : HttpCallback<SubmitChangeBatteryResponse.DataBean>() {
                    override fun onSucceed(data: SubmitChangeBatteryResponse.DataBean?) {
                        super.onSucceed(data)
                        // 换电成功
                        if (mView.subType() == 1) {
                            SendProgressActivity.start(context,1,mView.orderNo(),data?.id,2)
                            return
                        }
                        SendProgressActivity.start(context,1,mView.orderNo(),data?.id,1)
                    }
                }))
    }

    override fun getAfterSale() {

//        model?.getAfterSale(mView.bikeId(),object :HttpCallback<AfterSaleResponse.DataBean>(){
//            override fun onSucceed(data: AfterSaleResponse.DataBean?) {
//                super.onSucceed(data)
//                if (data?.bought == 0) {
//                    // 未购买
//                    mView.showBuyLayout()
//                } else if (data?.bought == 1) {
//                    // 已购买
//                    mView.showRenewalLayout(data.buttonText,data.placeholder,data.title)
//                }
//            }
//        })
    }

}