package com.baojia.mebike.feature.exclusive.personalChangeBattery

import android.bluetooth.BluetoothGatt
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.amap.api.mapcore.util.it
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseActivity
import com.baojia.mebike.buletooth.BluetoothCmdUtils
import com.baojia.mebike.buletooth.BluetoothManager
import com.baojia.mebike.buletooth.IBluetoothConfig
import com.baojia.mebike.buletooth.XiaoMiConfig
import com.baojia.mebike.config.Config
import com.baojia.mebike.dialog.LeftRightButtonTipsDialog
import com.baojia.mebike.dialog.WaitDialog
import com.baojia.mebike.util.CollectionsWrapper
import com.baojia.mebike.util.Toast
import com.clj.fastble.BleManager
import com.clj.fastble.callback.BleGattCallback
import com.clj.fastble.callback.BleNotifyCallback
import com.clj.fastble.callback.BleScanCallback
import com.clj.fastble.callback.BleWriteCallback
import com.clj.fastble.data.BleDevice
import com.clj.fastble.exception.BleException
import com.clj.fastble.scan.BleScanRuleConfig
import kotlinx.android.synthetic.main.activity_personal_change_battery.*

/**
 * +----------------------------------------------------------------------
 * | @time 2019/1/14:6:21 PM
 * | @author house
 * | @class_describe 自行换电页面
 * +----------------------------------------------------------------------
 **/
class PersonalChangeBatteryActivity : BaseActivity(), PersonalChangeBatteryContract.View {


    companion object {
        fun start(mContext: Context, bikeId: Int, isLock: Int = 0, lockDesc: String?) {
            val bundle = Bundle()
            bundle.putInt("bikeId", bikeId)
            bundle.putInt("isLock", isLock)
            bundle.putString("lockDesc", lockDesc)
            val intent = Intent(mContext, PersonalChangeBatteryActivity::class.java)
            intent.putExtras(bundle)
            mContext.startActivity(intent)
        }
    }

    private var bikeId: Int = 0
    // 是否支持后仓锁 0不支持
    private var isLock: Int = 0

    private var bluetoothResult: Int = 0
    private var lockDesc: String? = ""
    private var waitDialog: WaitDialog? = null


    private var mPresenter: PersonalChangeBatteryContract.Presenter? = null

    override fun layoutId(): Int {
        return R.layout.activity_personal_change_battery
    }

    override fun bindView(savedInstanceState: Bundle?) {

        bikeId = intent.getIntExtra("bikeId", 0)
        isLock = intent.getIntExtra("isLock", 0)
        lockDesc = intent.getStringExtra("lockDesc")

        mPresenter = PersonalChangeBatteryPresenter(this, this)

        setTitle("电池仓控制")
        setTitleBackground(R.color.white_color)

        setViewClickDuration(openCompartmentsButton, 1)
        setViewClickDuration(closeCompartmentsButton, 1)

        if (isLock != 0) {
            closeCompartmentsButton.visibility = View.VISIBLE
        }

        BleManager.getInstance().init(application)
        BleManager.getInstance()
                .enableLog(true)
                .setReConnectCount(3, 3000)
                .setConnectOverTime(3000).operateTimeout = 10000

        mPresenter?.getBoxInfo()
    }

    override fun setViewClick(view: View?) {
        super.setViewClick(view)
        when (view) {
            openCompartmentsButton -> {
                operation = "H"
                if (isCanConnectBle()) {
                    getCmd(3)
                    showLoading()
                    if (isConnect()) {
                        write()
                    } else {
                        startScan()
                    }
                    return
                }
                // 开仓
                mPresenter?.sendBox("H")
            }
            closeCompartmentsButton -> {
                operation = "HC"
                if (isCanConnectBle()) {
                    getCmd(4)
                    showLoading()
                    if (isConnect()) {
                        write()
                    } else {
                        startScan()
                    }
                    return
                }
                // 锁仓
                mPresenter?.sendBox("HC")
            }
        }
    }

    override fun isVisibleTitleBar(): Boolean {
        return true
    }

    override fun bikeId(): Int = bikeId

    private var tipsDialog:LeftRightButtonTipsDialog? = null

    override fun succeed(operation: String, message: String?) {
        if (TextUtils.equals(operation, "H")) {
            // "操作成功，使用完成后\n手动关闭电池仓即可"
            if(tipsDialog == null || tipsDialog?.dialog == null) {
                tipsDialog = LeftRightButtonTipsDialog.show(supportFragmentManager, null, lockDesc, R.mipmap.yellow_tips_icon)
            }
            if (tipsDialog?.dialog != null && !tipsDialog?.isAdded!!) {
                tipsDialog?.show(supportFragmentManager,"LeftRightButtonTipsDialog")
            }
        }
    }

    override fun setScanRule() {
        val scanRuleConfig = BleScanRuleConfig.Builder()
                .setDeviceMac(mPresenter?.boxMacAddress())           // 只扫描指定mac的设备，可选
                .setScanTimeOut(3000)              // 扫描超时时间，可选，默认10秒
                .build()
        BleManager.getInstance().initScanRule(scanRuleConfig)
    }

    private var cmd1: ByteArray? = null
    // B 锁车 A 启动
    private var operation: String = ""


    /**
     * 根据不同的车辆类型获取蓝牙指令
     *
     * @param type type 3:开锁 4：锁车
     */
    private fun getCmd(type: Int) {
        // 小蜜蜂
        cmd1 = BluetoothCmdUtils.xiaomiCmd(type)
    }


    private var bikeBleDevice: BleDevice? = null

    /**
     * 扫描
     */
    private fun startScan() {
        BleManager.getInstance().scan(object : BleScanCallback() {
            override fun onScanStarted(success: Boolean) {

            }

            override fun onScanning(bleDevice: BleDevice?) {

                if (bleDevice != null) {
                    if (TextUtils.equals(bleDevice.mac, mPresenter?.boxMacAddress())) {
                        // 扫描到了，开始连接
                        BleManager.getInstance().cancelScan()
                        bikeBleDevice = bleDevice
                        connect(bleDevice)
                    }
                }
            }

            override fun onScanFinished(scanResultList: List<BleDevice>) {
                if (CollectionsWrapper.isEmpty(scanResultList)) {
                    dismissLoading()
                    mPresenter?.sendBox(operation)
                }
            }
        })
    }

    private fun connect(bleDevice: BleDevice?) {
        BleManager.getInstance().connect(bleDevice, object : BleGattCallback() {

            override fun onStartConnect() {
                // 开始连接
            }

            override fun onConnectFail(bleDevice: BleDevice, exception: BleException) {
                // 连接失败
                if (waitDialog != null && waitDialog!!.isShowing) {
                    BleManager.getInstance().destroy()
                    dismissLoading()
                    setBluetoothSendFail()
                }
            }

            override fun onConnectSuccess(bleDevice: BleDevice, gatt: BluetoothGatt, status: Int) {
                // 连接成功
                setNotify()
                write()
            }

            override fun onDisConnected(isActiveDisConnected: Boolean, device: BleDevice, gatt: BluetoothGatt, status: Int) {
                // 断开连接
                if (waitDialog != null && waitDialog!!.isShowing) {
                    dismissLoading()
                    mPresenter?.sendBox(operation)
                }

            }
        })
    }


    private fun setNotify() {
        BleManager.getInstance().notify(bikeBleDevice, getBluetoothConfig()!!.uuidService.toString(), getBluetoothConfig()!!.uuidNotifyRead.toString(), object : BleNotifyCallback() {
            override fun onNotifySuccess() {
                println("setNotify成功====")
                write()
            }

            override fun onNotifyFailure(exception: BleException) {
                println("setNotify失败====")
                //                BleManager.getInstance().destroy();
            }

            override fun onCharacteristicChanged(data: ByteArray?) {
                dismissLoading()
                if (data == null) {
                    return
                }
                // 小蜜数据解析
                var length: Byte = 0
                if (data!!.size > 1) {
                    length = data[1]
                }
                if (data.size > 1 + length) {
                    if (data[1 + length].toInt() == 0) {
                        bluetoothResult = 1
//                        mPresenter.saveBluetoothLog()
                        if (TextUtils.equals("H", operation)) {
                            succeed(operation, lockDesc)
                        }
                        Toast.show(this@PersonalChangeBatteryActivity, "蓝牙操作成功")
                    } else {
                        bluetoothResult = 0
//                        mPresenter.saveBluetoothLog()
                        Toast.show(this@PersonalChangeBatteryActivity, "蓝牙操作失败")
                        setBluetoothSendFail()
                    }
                }
            }
        })

    }

    /**
     * 设置蓝牙发送指令失败判断逻辑
     */
    private fun setBluetoothSendFail() {
        mPresenter?.sendBox(operation)
    }

    private fun getBluetoothConfig(): IBluetoothConfig? {
        var bluetoothConfig: IBluetoothConfig? = null
        // 小蜜蜂
        bluetoothConfig = XiaoMiConfig()
        return bluetoothConfig
    }

    private fun write() {
        BleManager.getInstance().write(bikeBleDevice, getBluetoothConfig()!!.uuidService.toString(), getBluetoothConfig()!!.uuidNotify.toString(),
                cmd1, object : BleWriteCallback() {
            override fun onWriteSuccess(current: Int, total: Int, data: ByteArray) {
                println("执行成功====")
                dismissLoading()
            }

            override fun onWriteFailure(exception: BleException) {
                dismissLoading()
                mPresenter?.sendBox(operation)
            }
        })
    }

    /**
     * 是否可以使用蓝牙
     * @return
     */
    private fun isCanConnectBle(): Boolean {
        return !TextUtils.isEmpty(mPresenter?.boxMacAddress()) && BluetoothManager.isBluetoothSupported() && BluetoothManager.isBluetoothEnabled()
    }


    /**
     * 蓝牙是否已连接
     * @return
     */
    private fun isConnect(): Boolean {
        return bikeBleDevice != null && BleManager.getInstance().isConnected(bikeBleDevice)
    }

    private fun showLoading() {
        if (waitDialog == null) {
            waitDialog = WaitDialog(this)
            waitDialog!!.setCancelable(false)
            waitDialog!!.setCanceledOnTouchOutside(false)
        }
        if (!waitDialog!!.isShowing) {
            waitDialog!!.show()
        }
    }

    private fun dismissLoading() {
        waitDialog?.let {
            it.dismiss()
        }
    }


    override fun onDestroy() {
        super.onDestroy()
        BleManager.getInstance().disconnectAllDevice()
        BleManager.getInstance().destroy()
    }

    override fun setPresenter(presenter: PersonalChangeBatteryContract.Presenter?) {
        this.setIPresenter(presenter)
    }

}