package com.baojia.mebike.feature.exclusive.shopping.nearBike

import android.app.Activity
import com.baojia.mebike.base.aboutbike.AboutBikeModel
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.callback.SubmitOrderCallback
import com.baojia.mebike.data.response.BaseResponse
import com.baojia.mebike.data.response.bike.ProcessingOrderResponse
import com.baojia.mebike.data.response.bike.SubmitOrderResponse
import com.baojia.mebike.data.response.main.SearchBikeResponse
import com.baojia.mebike.http.HttpUtils
import com.baojia.mebike.util.Toast
import io.reactivex.disposables.Disposable
import java.util.*

/**
 * +----------------------------------------------------------------------
 * | @time 2018/12/27:8:16 PM
 * | @author house
 * | @class_describe
 * +----------------------------------------------------------------------
 **/
class NearBikeModel constructor(mContext: Activity) : AboutBikeModel(mContext) {
    // 个人专属搜索附近车辆
    private val PERSON_NEAR_BIKE = "/bike/knight/search/personal/near/bike/v2"
    // 个人专属预约车辆
    private val PERSON_BIKE_APPOINTMENT = "/pmall/bike/reserve/v3"
    // 取消预约
    private val PERSON_CANCEL_APPOINTMENT = "/pmall/order/reserve/cancel/v3"

    private val PERSON_ORDER_PROGRESS = "/pmall/order/status/v3"

    /**
     * 查看正在进行中订单
     *
     * @param httpCallback
     * @return
     */
    fun getProcessingOrder(orderNo: String?, httpCallback: HttpCallback<ProcessingOrderResponse.DataBean>?): Disposable {
        val params = HashMap<String, Any?>()
        params["orderNo"] = orderNo
        return HttpUtils.postRequest(activity, PERSON_ORDER_PROGRESS, params,
                object : HttpCallback<ProcessingOrderResponse>() {

                    override fun onSucceed(data: ProcessingOrderResponse) {
                        super.onSucceed(data)

                        if (data.data == null) {
                            httpCallback?.onFailed(data.code, data.message)
                            return
                        }

                        httpCallback?.onSucceed(data.data)
                    }

                    override fun onFailed(code: Int, message: String) {
                        super.onFailed(code, message)
                        Toast.show(activity, message)
                    }

                    override fun onError(code: Int, message: String) {
                        super.onError(code, message)
                        onFailed(code, message)
                    }
                }, ProcessingOrderResponse::class.java)
    }


    /**
     * 个人专属车搜索附近车辆
     *
     * @param prodId 个人专属车产品ID
     * @param personalCarFlag 个人专属车,搜索类型（如果为空或者0,代表是其他搜索方式）购买时 1，换车时 2
     */
    fun searchNearBike(latitude: Double?, longitude: Double?, zoomLevel: Int,
                       httpCallback: HttpCallback<SearchBikeResponse.DataBean>?,
                       prodId: Long = 0, colorId: Int = 0, personalCarFlag: Int = 1): Disposable {
        val params = mutableMapOf<String, Any?>()
        params["latitude"] = latitude
        params["longitude"] = longitude
        params["zoomLevel"] = zoomLevel
        params["personalCarFlag"] = personalCarFlag

        if (prodId > 0) {
            params["prodId"] = prodId
        }
        if (colorId > 0) {
            params["colorId"] = colorId
        }
        return HttpUtils.postRequest(activity, PERSON_NEAR_BIKE, params, object : HttpCallback<SearchBikeResponse>() {
            override fun onSucceed(data: SearchBikeResponse?) {
                super.onSucceed(data)
                httpCallback?.onSucceed(data?.data)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                Toast.show(activity, message)
                httpCallback?.onFailed(code, message)
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code, message)
            }
        },
                SearchBikeResponse::class.java)
    }

    /**
     * 预约
     */
    fun appointment(plateNo: String?, orderNo: String?, submitOrderCallback: SubmitOrderCallback?): Disposable {
        val params = mutableMapOf<String, Any?>()
        params["orderNo"] = orderNo
        params["plateNo"] = plateNo
        return HttpUtils.postRequest(activity, PERSON_BIKE_APPOINTMENT, params, object : HttpCallback<SubmitOrderResponse>() {
            override fun onSucceed(data: SubmitOrderResponse?) {
                super.onSucceed(data)
                if (data?.data == null) {
                    submitOrderCallback?.appointmentSucceed(null, 0)
                    return
                }
                submitOrderCallback?.appointmentSucceed(data?.data.orderNo,
                        data?.data.remainSecond)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                Toast.show(activity, message)
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code, message)
            }
        },
                SubmitOrderResponse::class.java)
    }

    /**
     * 取消预约
     */
    fun cancelAppointment(orderNo: String?, httpCallback: HttpCallback<BaseResponse>?): Disposable {
        val params = mutableMapOf<String, Any?>()
        params["orderNo"] = orderNo
        return HttpUtils.postRequest(activity, PERSON_CANCEL_APPOINTMENT, params, httpCallback, BaseResponse::class.java)
    }

}