package com.baojia.mebike.feature.exclusive.shopping.near_shop

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View
import com.amap.api.maps.model.Marker
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseActivity
import com.baojia.mebike.data.response.exclusive.NearShopResponse
import com.baojia.mebike.dialog.SelectMapDialog
import com.baojia.mebike.map.MapCallback
import com.baojia.mebike.util.DensityUtil
import com.baojia.mebike.util.JumpActivity
import com.baojia.mebike.util.ToolBox
import kotlinx.android.synthetic.main.activity_near_shop.*


/**
 * +----------------------------------------------------------------------
 * | @time 2018/12/20:3:50 PM
 * | @author house
 * | @class_describe 查看附近门店
 * +----------------------------------------------------------------------
 **/
class NearShopActivity : BaseActivity(), NearShopContract.View, MapCallback.OnMarkerClickListener,
        MapCallback.OnMapCancelListener, View.OnClickListener {


    companion object {

        fun start(context: Activity) {
            val intent = Intent(context, NearShopActivity::class.java)
            context.startActivity(intent)
        }
    }

    private var mPresenter: NearShopContract.Presenter? = null


    private var mapViewImp: NearShopMapViewImp? = null
    private val siteList: ArrayList<NearShopResponse.DataBean.StoresBean> = arrayListOf()

    private var curSelectData: NearShopResponse.DataBean.StoresBean? = null

    override fun layoutId(): Int {
        return R.layout.activity_check_near_power_station
    }

    override fun bindView(savedInstanceState: Bundle?) {

        titleView.text = "查看附近门店"

        mapView.onCreate(savedInstanceState)
        mapViewImp = NearShopMapViewImp(mapView, this)

        mapViewImp?.let {
            it.mViewPaddingBottom = DensityUtil.getViewHeight(navigationBackgroundImageView) + DensityUtil.dip2px(this, 15f)
            it.addLocation()
            it.moveToLocationLatLng(0, 0)
            it.setOnMapCancelListener(this)
            it.setOnMarkerClickListener(this)
        }

        mPresenter = NearShopPresenter(this, this)


        callButton.setOnClickListener(this)
        backButton.setOnClickListener(this)
        navigationButton.setOnClickListener(this)
        navigationBackgroundImageView.setOnClickListener(this)

        mPresenter?.searchPowerStation()

    }

    override fun onResume() {
        super.onResume()
        mapViewImp?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapViewImp?.onPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        mapViewImp?.onDestroy()
    }


    /**
     * 显示底部导航信息
     */
    private fun setNavigationInfoVisible(isShow: Boolean) {
        var isVisible: Int
        if (isShow) {
            isVisible = View.VISIBLE
        } else {
            isVisible = View.GONE
        }
        navigationBackgroundImageView.visibility = isVisible
        locationIconImageView.visibility = isVisible
        navigationButton.visibility = isVisible
        callButton.visibility = isVisible
        addressTextView.visibility = isVisible
        distanceTextView.visibility = isVisible
        lineImageView.visibility = isVisible
        navigationIntroduceTextView.visibility = isVisible
    }


    override fun setPresenter(presenter: NearShopContract.Presenter?) {
        this.setIPresenter(presenter)
    }

    override fun drawSitMarker(markerBeanList: ArrayList<NearShopResponse.DataBean.StoresBean>?) {
        mapViewImp?.addSiteMarker(markerBeanList)

    }

    override fun onMarkerClick(marker: Marker?) {
        // 底部列表隐藏，底部信息栏显示
        setNavigationInfoVisible(true)
        curSelectData = marker?.`object` as NearShopResponse.DataBean.StoresBean

        curSelectData?.let {
            addressTextView.text = it.doorStoreName
            navigationIntroduceTextView.text = it.doorStoreAddress
            if (it.distance >= 1000) {
                distanceTextView.text = "${ToolBox.getTwoDecimal(it.distance / 1000)}km"
            } else {
                distanceTextView.text = "${ToolBox.getTwoDecimal(it.distance)}m"
            }
        }
    }


    override fun onMapClick(marker: Marker?) {
        // 清除路线规划，底部列表显示
        setNavigationInfoVisible(false)
    }

    override fun onClick(v: View?) {
        when (v) {
            callButton -> {
                JumpActivity.JumpToCallPhone(this, curSelectData?.linkmanPhone)
            }
            navigationButton -> {
                SelectMapDialog.show(this, supportFragmentManager, curSelectData?.latitude
                        ?: 0.0, curSelectData?.longitude ?: 0.0)
            }
            backButton -> {
                finish()
            }
            navigationBackgroundImageView -> {

            }
        }
    }
}