package com.baojia.mebike.feature.join.JoinBuyActivity

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.amap.api.mapcore.util.it
import com.baojia.mebike.R
import com.baojia.mebike.alipay.AliPayManager
import com.baojia.mebike.base.BaseActivity
import com.baojia.mebike.callback.IDialogCallback
import com.baojia.mebike.callback.PayListener
import com.baojia.mebike.data.response.order.PayByOtherResponse
import com.baojia.mebike.dialog.CommonTipsDialog
import com.baojia.mebike.dialog.LeftRightButtonTipsDialog
import com.baojia.mebike.feature.join.CopyDownloadUrlDialog
import com.baojia.mebike.feature.join.income.JoinIncomeActivity
import com.baojia.mebike.feature.main.MainActivity
import com.baojia.mebike.util.AppManager
import com.baojia.mebike.util.JumpActivity
import com.baojia.mebike.util.Toast
import com.baojia.mebike.util.WechatPayUtils
import com.baojia.mebike.widget.CommonWebViewActivity
import kotlinx.android.synthetic.main.activity_join_pay.*
import kotlinx.android.synthetic.main.activity_join_pay_new.*
import kotlinx.android.synthetic.main.include_bottom_common.*
import kotlinx.android.synthetic.main.include_bottom_common.view.*
import kotlinx.android.synthetic.main.include_payment.*
import java.util.jar.Attributes

class JoinBuyActivity : BaseActivity(), JoinBuyContract.View, PayListener {

    companion object {
        @JvmStatic
        fun start(activity: Activity, contractRealName: String? = "", contractIdCard: String? = "", joinMoney: String? = "", contractMobile: String? = "", orderNo: String? = "") {
            val intent = Intent(activity, JoinBuyActivity::class.java)
            intent.putExtra("joinMoney", joinMoney)
            intent.putExtra("contractMobile", contractMobile)
            intent.putExtra("contractRealName", contractRealName)
            intent.putExtra("contractIdCard", contractIdCard)
            intent.putExtra("orderNo", orderNo)
            activity.startActivity(intent)
        }
    }

    private var joinMoney = ""
    private var phone = ""
    private var orderNo = ""
    private var contractRealName = ""
    private var contractIdCard = ""
    private var fadadaSignUrl = ""

    override fun layoutId(): Int = R.layout.activity_join_pay_new

    override fun isVisibleTitleBar(): Boolean = false
    // 支付方式 1：微信支付 2：支付宝支付
    private var payMethod = 1
    val presenter = JoinBuyPresenter(this, this)
    override fun bindView(savedInstanceState: Bundle?) {

        intent?.let {
            joinMoney = it.getStringExtra("joinMoney")
            contractRealName = it.getStringExtra("contractRealName")
            phone = it.getStringExtra("contractMobile")
            contractIdCard = it.getStringExtra("contractIdCard")
            orderNo = it.getStringExtra("orderNo")
        }

        topMoneyTv.text = "￥$joinMoney"
        phoneNumberEt.setText(phone)
        realNameEt.setText(contractRealName)
        idCardEt.setText(contractIdCard)

        //如果已经实名则不可改动
        if (!TextUtils.isEmpty(contractRealName)) {
            realNameEt.isEnabled = false
        }
        if (!TextUtils.isEmpty(contractIdCard)) {
            idCardEt.isEnabled = false
        }

        setPayMethod(1)

        setViewClickDuration(wechatPayLayout, 1)
        setViewClickDuration(aliPayLayout, 1)
        setViewClickDuration(confirmPayTv, 1)
        setViewClickDuration(topBackgroundIv, 1)

    }

    override fun setViewClick(view: View?) {
        super.setViewClick(view)
        when (view) {
            wechatPayLayout -> setPayMethod(1)
            aliPayLayout -> setPayMethod(2)
            confirmPayTv -> {
                if (isNull()) {
                    return
                }
                presenter.joinPay()
            }
            topBackgroundIv ->{
                finish()
            }
        }
    }

    override fun onResume() {
        super.onResume()
        presenter.checkOrderPayStatus()
    }

    private fun isNull(): Boolean {
        if (realNameEt.text.toString().isEmpty()) {
            Toast.show(this, "请输入您的姓名")
            return true
        }
        if (idCardEt.text.toString().isEmpty()) {
            Toast.show(this, "请输入您的身份证号")
            return true
        }
        if (phoneNumberEt.text.toString().isEmpty()) {
            Toast.show(this, "请输入您的手机号")
            return true
        }

        return false
    }


    private fun setPayMethod(payWay: Int) {
        if (payWay == 1) {
            // 微信支付
            wechatSelectImageView.setImageResource(R.mipmap.selected_icon)
            aliSelectImageView.setImageResource(R.mipmap.unselected_icon)
        } else {
            // 支付宝支付
            aliSelectImageView.setImageResource(R.mipmap.selected_icon)
            wechatSelectImageView.setImageResource(R.mipmap.unselected_icon)
        }
        payMethod = payWay
    }

    override fun setPresenter(presenter: JoinBuyContract.Presenter?) {
    }


    override fun orderPaySucceed(dataBean: PayByOtherResponse.DataBean) {
        if (dataBean.payChannelId == 1) {
            WechatPayUtils.wechatPay(this, dataBean.appid, dataBean.partnerid, dataBean.prepayid,
                    dataBean.wxpackage, dataBean.noncestr, dataBean.timestamp,
                    dataBean.sign)

        } else if (dataBean.payChannelId == 2) {
            AliPayManager(this)
                    .alipayV2(this, dataBean.orderInfo)
        }
    }

    override fun onSuccess() {
    }

    override fun onFailed(msg: String?) {
        Toast.show(this, msg)
    }

    override fun payMethod(): Int = payMethod

    override fun orderNo(): String = orderNo

    override fun getName(): String = realNameEt.text.toString()

    override fun contractIdCard(): String = idCardEt.text.toString()

    override fun getMobile(): String = phoneNumberEt.text.toString()

    override fun checkPaySuccess(url: String) {
        fadadaSignUrl = url
       var dialog =  LeftRightButtonTipsDialog.showUnCancle( supportFragmentManager, "", "支付成功,请签署相关协议", 2,  R.drawable.icon_tips_succeed,"查看收益","立即签署",false)
        dialog.setDialogCallback(object :IDialogCallback(){
            override fun onConfirm() {
                super.onConfirm()
                AppManager.getInstance().finishActivity(CommonWebViewActivity::class.java)
                this@JoinBuyActivity.finish()
                JumpActivity.JumpContractWebView(this@JoinBuyActivity,fadadaSignUrl)
            }

            override fun onCancel() {
                super.onCancel()
                AppManager.getInstance().finishActivity(CommonWebViewActivity::class.java)
                this@JoinBuyActivity.finish()
                JoinIncomeActivity.start(this@JoinBuyActivity)
            }


        })

    }

}