package com.baojia.mebike.feature.join.JoinBuyActivity

import android.app.Activity
import android.text.TextUtils
import com.baojia.mebike.R
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.data.response.BaseResponse
import com.baojia.mebike.data.response.DataStringResponse
import com.baojia.mebike.data.response.join.JoinCommitResponse
import com.baojia.mebike.data.response.join.PhoneRealNameResponse
import com.baojia.mebike.data.response.order.PayByOtherResponse
import com.baojia.mebike.feature.model.PayModel
import com.baojia.mebike.http.HttpUtils
import com.baojia.mebike.util.Toast
import io.reactivex.disposables.Disposable
import java.util.*

open class JoinBuyModel(activity: Activity) : PayModel(activity) {
    // 支付接口
    private val JOIN_ORDER_PAY = "/join/joinOrder/payMoney"
    // 查询订单支付状态
    private val CHECK_ORDER_PAY_STATUS_JOIN = "/join/joinOrder/getPayResult"
    //意向加盟提交
    private val JOIN_COMMIT = "/join/create/joinOrder"
    //姓名手机号
    private val JOIN_NAME_PHONE = "/join/get/join/userMessage"


    /**
     * 加盟支付
     */
    fun orderPay(name: String,mobile: String,contractIdCard: String,payMethod: Int,orderNo: String, httpCallback: HttpCallback<PayByOtherResponse.DataBean>?): Disposable {

        val params = HashMap<String, Any>()
        params["contractRealName"] = name
        params["contactMobile"] = mobile
        params["contractIdCard"] = contractIdCard
        params["payMethod"] = payMethod
        params["paymentType"] = 14
        params["orderNo"] = orderNo
        return HttpUtils.postRequest(activity, JOIN_ORDER_PAY, params, object : HttpCallback<PayByOtherResponse>() {
            override fun onSucceed(data: PayByOtherResponse?) {
                super.onSucceed(data)
                data?:return
                if (data?.data == null) {
                    Toast.show(activity, R.string.json_error)
                    return
                }
                httpCallback?.onSucceed(data.data)
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)
                Toast.show(activity, message)
                if (code == 121) {
                    // 已支付
                    httpCallback?.onFailed(code, message)
                } else if (code == 187) {
                    // 已充值
                }
            }

            override fun onError(code: Int, message: String) {
                super.onError(code, message)
                onFailed(code, message)
            }
        }, PayByOtherResponse::class.java)
    }

    fun joinCommit(name: String,mobile: String,productId: Int,year: Int, httpCallback: HttpCallback<JoinCommitResponse>?): Disposable? {
        val params = HashMap<String, Any>()
        params["name"] = name
        params["mobile"] = mobile
//        params["productId"] = productId
//        params["year"] = year
        return HttpUtils.postRequest(activity, JOIN_COMMIT, params, true, object : HttpCallback<JoinCommitResponse>() {
            override fun onSucceed(data: JoinCommitResponse?) {
                super.onSucceed(data)
                data?.let {
                    httpCallback?.onSucceed(it)
                }
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)
                Toast.show(activity, message)
            }

            override fun onError(code: Int, message: String) {
                super.onError(code, message)
                onFailed(code, message)
            }
        }, JoinCommitResponse::class.java)
    }

    fun getNamePhone( httpCallback: HttpCallback<PhoneRealNameResponse?>?): Disposable? {
        return HttpUtils.postRequest(activity, JOIN_NAME_PHONE, null, true, object : HttpCallback<PhoneRealNameResponse>() {
            override fun onSucceed(data: PhoneRealNameResponse) {
                super.onSucceed(data)
                data?.let {
                    httpCallback?.onSucceed(it)
                }
            }
        }, PhoneRealNameResponse::class.java)
    }


    /**
     * 查询订单支付状态
     *
     * @param orderNo
     * @param orderType    查询的订单类型，3 加盟查单
     * @param httpCallback
     * @return
     */
    fun checkJoinPayStatus(orderNo: String, orderType: Int, httpCallback: HttpCallback<DataStringResponse>?): Disposable? {
        if (TextUtils.isEmpty(orderNo)) {
            return null
        }
        val params = HashMap<String, Any>()
        params["orderNo"] = orderNo
        if (orderType >= 0) {
            params["orderType"] = orderType
        }
        return HttpUtils.postRequest(activity, CHECK_ORDER_PAY_STATUS_JOIN, params, true, object : HttpCallback<DataStringResponse>() {
            override fun onSucceed(data: DataStringResponse) {
                super.onSucceed(data)
                // 订单已支付
                // 支付成功
                httpCallback?.onSucceed(data)
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)
                if (code == 174) {
                    return
                }
                httpCallback?.onFailed(code,message)
            }

            override fun onError(code: Int, message: String) {
                super.onError(code, message)
                onFailed(code, message)
            }
        }, DataStringResponse::class.java)
    }

}