package com.baojia.mebike.feature.join.contract

import android.os.Bundle
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseCommonAdapter
import com.baojia.mebike.base.BaseRefreshActivity
import com.baojia.mebike.callback.IDialogCallback
import com.baojia.mebike.data.response.join.JoinContractResponse
import com.baojia.mebike.dialog.CommonTipsDialog
import com.baojia.mebike.feature.infinitecard.dialog.InputInviteCodeDialog
import com.baojia.mebike.feature.join.CopyDownloadUrlDialog
import com.baojia.mebike.feature.join.income.JoinIncomeActivity
import com.baojia.mebike.util.AppManager
import com.baojia.mebike.util.JumpActivity
import com.baojia.mebike.util.ToolBox
import com.baojia.mebike.util.recyclerview.RecycleViewDivider
import com.baojia.mebike.widget.CommonWebViewActivity
import kotlinx.android.synthetic.main.layout_recyclerview_only.*

/**
 * +----------------------------------------------------------------------
 * | @time 2019/4/12:3:17 PM
 * | @author house
 * | @class_describe 合同列表页面   列表xml---item_contract.xml
 * +----------------------------------------------------------------------
 **/
class ContractActivity : BaseRefreshActivity<ContractContract.Presenter, ContractContract.View>(), ContractContract.View {

    private val mData = arrayListOf<JoinContractResponse.DataBean>()
    private var mAdapter: ContractAdapter? = null
    private var mPresenter: ContractContract.Presenter? = null

    override fun setData(data: ArrayList<JoinContractResponse.DataBean>, isRefresh: Boolean) {
        if (isRefresh) {
            mData.clear()
        }
        mData.addAll(data)
        mAdapter?.notifyDataSetChanged()
    }


    override fun bindView(savedInstanceState: Bundle?) {
        super.bindView(savedInstanceState)
        mPresenter = ContractPresenter(this, this)
    }

    override fun setCenterTitle(): String = "我的合同"

    override fun initAdapter() {
        mAdapter = ContractAdapter(this, mData, supportFragmentManager)
        recyclerview.adapter = mAdapter


        val manager = LinearLayoutManager(this)
        recyclerview.layoutManager = manager
        recyclerview.setHasFixedSize(true)
        recyclerview.setBackgroundColor(ToolBox.getColor(R.color.background_color))
        val padding = ToolBox.dip2px(10f);
        recyclerview.setPadding(padding, padding, padding, 0)
        recyclerview.addItemDecoration(RecycleViewDivider(this, RecyclerView.VERTICAL, ToolBox.dip2px(10f), R.color.background_color))

        mAdapter?.onViewClickListener = object : ContractAdapter.OnViewClickListener {
            override fun onClick(clickType: Int, position: Int, itemPosition: Int) {
                when (clickType) {
                    1 -> {
                        var dialog = InputInviteCodeDialog()
                        val bundle = Bundle()
                        bundle.putInt("intentType", 2)
                        bundle.putString("orderNo",mData[position].orderNo)
                        bundle.putInt("type",mData[position].contractInfos[itemPosition].type)
                        dialog.arguments = bundle
                        dialog.show(supportFragmentManager, "InputInviteCodeDialog")
                    }
                    2 -> {
                        // 下载
                        val copyDialog = CopyDownloadUrlDialog()
                        val bundle = Bundle()
                        bundle.putString("downloadUrl", mData[position].contractInfos[itemPosition].downUrl)
                        copyDialog.arguments = bundle
                        copyDialog.show(supportFragmentManager, "CopyDownloadUrlDialog")
                    }
                    0 -> {
                        // 跳转页面
                        if (mData[position].authStatus == 0) {
                            // 实名认证弹窗
                             CommonTipsDialog.showDialog(this@ContractActivity,supportFragmentManager,"","请先完成实名认证","立即认证","",R.mipmap.yellow_tips_icon,object :IDialogCallback(){
                                 override fun onConfirm() {
                                     super.onConfirm()
                                    JumpActivity.JumpCertificateActivity(this@ContractActivity,2,"")
                                 }

                                 override fun onCancel() {
                                     super.onCancel()
                                 }
                             })
                        } else {
                            // 跳转合同页面
                            if (mData[position].contractInfos[itemPosition].status == 1||mData[position].contractInfos[itemPosition].status == 2) {
                                JumpActivity.JumpWebView(this@ContractActivity,"",mData[position].contractInfos[itemPosition].url)
                            } else if (mData[position].contractInfos[itemPosition].status == 0) {
                                JumpActivity.JumpContractWebView(this@ContractActivity,mData[position].contractInfos[itemPosition].signUrl)
                            }
                        }

                    }

                }
            }

        }
    }

    override fun onRefresh() {
        mPresenter?.onRefresh()
    }

    override fun getAdapter(): BaseCommonAdapter<Any?> = mAdapter as BaseCommonAdapter<Any?>

    override fun onLoadMore() {
        mPresenter?.onLoadMore()
    }

    override fun isVisibleTitleBar(): Boolean {
        return true
    }

    override fun onResume() {
        super.onResume()
        refreshLayout.autoRefresh()
    }
}