package com.baojia.mebike.feature.join.details

import android.app.Activity
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.data.response.BaseResponse
import com.baojia.mebike.data.response.join.JoinOrderDetailsResponse
import com.baojia.mebike.feature.model.PayModel
import com.baojia.mebike.http.HttpUtils
import com.baojia.mebike.util.Toast
import io.reactivex.disposables.Disposable

/**
 * +----------------------------------------------------------------------
 * | @time 2018/12/14:6:08 PM
 * | @author house
 * | @class_describe
 * +----------------------------------------------------------------------
 **/
class JoinDetailsModel constructor(val mContext: Activity) : PayModel(mContext) {
    // 订单详情
    private val ORDER_DETAILS_URL = "/join/order/detail"
    // 提前出售车辆
    private val SELL_URL = "/join/advance/sell"
    //QUXIAO取消订单
    private val CANCLE_URL = "/join/cancel/join/order"

    /**
     * 获取订单详情
     */
    fun getOrderDetails(orderNo: String, httpCallback: HttpCallback<JoinOrderDetailsResponse.DataBean>?): Disposable {

        val params: Map<String, Any> = mapOf("orderNo" to orderNo)

        return HttpUtils.postRequest(mContext, ORDER_DETAILS_URL, params, object : HttpCallback<JoinOrderDetailsResponse>() {
            override fun onSucceed(data: JoinOrderDetailsResponse?) {
                super.onSucceed(data)
                data?:return
                data?.data ?: return

                httpCallback?.onSucceed(data.data)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                Toast.show(mContext, message)
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code, message)
            }
        }, JoinOrderDetailsResponse::class.java)

    }

    /**
     * 提前出售车辆
     */
    fun sellBike(orderNo:String,httpCallback: HttpCallback<BaseResponse>):Disposable {
        val params: Map<String, Any> = mapOf("orderNo" to orderNo)
        return HttpUtils.postRequest(mContext,SELL_URL,params,object : HttpCallback<BaseResponse>(){
            override fun onSucceed(data: BaseResponse?) {
                super.onSucceed(data)
                httpCallback.onSucceed(null)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                Toast.show(mContext,message)
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code,message)
            }
        },BaseResponse::class.java)
    }

    /**
     * quxiao订单
     */
    fun cancleOrder(orderNo:String,httpCallback: HttpCallback<BaseResponse>):Disposable {
        val params: Map<String, Any> = mapOf("orderNo" to orderNo)
        return HttpUtils.postRequest(mContext,CANCLE_URL,params,object : HttpCallback<BaseResponse>(){
            override fun onSucceed(data: BaseResponse?) {
                super.onSucceed(data)
                httpCallback.onSucceed(null)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                Toast.show(mContext,message)
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code,message)
            }
        },BaseResponse::class.java)
    }


}