package com.baojia.mebike.feature.person_company.bike_repair

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.View
import androidx.recyclerview.widget.GridLayoutManager
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseActivity
import com.baojia.mebike.callback.PhotoDialogCallback
import com.baojia.mebike.data.response.company_personal.BikeRepairLocationResponse
import com.baojia.mebike.dialog.ClearAndAddPicDialog
import com.baojia.mebike.feature.adapter.PicShowAdapter
import com.baojia.mebike.feature.exclusive.shopping.renewalfee.RenewalFeeAct
import com.baojia.mebike.feature.exclusive.shopping.repairlist.RepairListActivity
import com.baojia.mebike.util.*
import com.baojia.mebike.util.recyclerview.GridSpacingItemDecoration
import com.yanzhenjie.album.Album
import id.zelory.compressor.Compressor
import kotlinx.android.synthetic.main.activity_company_personal_bike_repair.*
import kotlinx.android.synthetic.main.include_bottom_common_one_button.*
import kotlinx.android.synthetic.main.include_renewal_combo.*
import java.io.File
import java.io.IOException
import java.util.*

/**
 * +----------------------------------------------------------------------
 * | @time 2019/1/15:2:38 PM
 * | @author house
 * | @class_describe 维修保养页面
 * +----------------------------------------------------------------------
 **/
class BikeRepairActivity : BaseActivity(),BikeRepairContract.View {


    companion object {
        @JvmStatic
        fun start(activity: Context,pmallOrderId :Int?,bikeId :Int?,virGroupId:Long? = 0){
            var intent = Intent(activity,BikeRepairActivity::class.java)
            intent.putExtra("pmallOrderId",pmallOrderId)
            intent.putExtra("bikeId",bikeId)
            intent.putExtra("virGroupId",virGroupId)
            activity.startActivity(intent)
        }
    }

    private var bikeId: Int = 0
    private var pmallOrderId: Int = -1
    private var virGroupId: Long = 0

    private val TAKEPHOTO_CODE_INFORM = 1002
    private var advice = ""
    private var mPicShowAdapter: PicShowAdapter? = null
    //拍照返回的图片列表
    private val piclist = ArrayList<String>()
    //压缩图片后的Filelist
    private val fileList = arrayListOf<File?>()
    //重新拍照事件的点击图片位置
    private var position_retake: Int = 0
    //是否为重新拍照事件
    private var isReTakePhoto = false
    private var actualImage: File? = null
    private var compressedImage: File? = null

    private val repairLocationList = arrayListOf<BikeRepairLocationResponse>()
    private var bikeRepairAdapter: BikeRepairLocationAdapter? = null
    private var presenter :BikeRepairPresenter? = null
    private var noService = true

    override fun layoutId(): Int = R.layout.activity_company_personal_bike_repair

    override fun isVisibleTitleBar(): Boolean = true

    override fun setTitle(): String = "维修保养"



    override fun bindView(savedInstanceState: Bundle?) {

        intent?.let {
            bikeId = it.getIntExtra("bikeId",0)
            virGroupId = it.getLongExtra("virGroupId",virGroupId)
            pmallOrderId = it.getIntExtra("pmallOrderId",-1)
        }

        LogUtil.ee("$pmallOrderId ---")

        setLocationRecyclerView()
        setOther()

        presenter = BikeRepairPresenter(this,this)

        confirmButton.text = "上报故障"
        setViewClickDuration(confirmButton,1)
        setViewClickDuration(buyButton,1)
        setViewClickDuration(renewalButton,1)
        SoftHideKeyBoardUtil.assistActivity(this)
    }

    override fun setRightButtonText(): String = "记录"

    override fun setRightButtonClickListener(view: View?) {
        super.setRightButtonClickListener(view)
        RepairListActivity.start(this,bikeId)
    }

    override fun setViewClick(view: View?) {
        super.setViewClick(view)
        when (view) {
            confirmButton -> {
                if (!ToolBox.isCompanyExclusive() && noService){
                    return
                }
                if (damagedPart().isEmpty()){
                    Toast.show(this,"请选择损坏部位")
                    return
                }
                presenter?.uploadPhoto()
            }
            buyButton, renewalButton -> {
                // 购买、续期 跳转同一页面
                RenewalFeeAct.start(this,4,bikeId,pmallOrderId,virGroupId.toInt())
            }
        }
    }

    private fun setOther() {

        val gridLayoutManager = GridLayoutManager(this, 4)
        pictureRecyclerView.layoutManager = gridLayoutManager
        pictureRecyclerView.isNestedScrollingEnabled = false
        mPicShowAdapter = PicShowAdapter(this, piclist)
        pictureRecyclerView.adapter = mPicShowAdapter

        mPicShowAdapter?.setOnPicAddAndClearClickLinstener { position ->
            var clearAndAddPicDialog: ClearAndAddPicDialog? = ClearAndAddPicDialog(this)
            clearAndAddPicDialog?.setShow()
            clearAndAddPicDialog?.setPhotoDialogCallback(object : PhotoDialogCallback {
                override fun onPhotoButton(value: String?) {
                    piclist.removeAt(position)
                    fileList.removeAt(position)
//                    setInformButtonState()
                    mPicShowAdapter?.notifyDataSetChanged()
                }

                override fun onSelectButton(value: String?) {
                    isReTakePhoto = true
                    position_retake = position
                    Album.camera(this@BikeRepairActivity).start(TAKEPHOTO_CODE_INFORM)
                }

                override fun onCancleButton(value: String) {}
            })
        }

        mPicShowAdapter?.setOnPicAddClickLinstener {
            isReTakePhoto = false
            Album.camera(this@BikeRepairActivity).start(TAKEPHOTO_CODE_INFORM)
        }

        mPicShowAdapter?.setOnPicClearClickLinstener { }

        repairIntroduceEditText.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(charSequence: CharSequence, i: Int, i1: Int, i2: Int) {}

            override fun onTextChanged(charSequence: CharSequence, i: Int, i1: Int, i2: Int) {
                textCountTextView.text = "${i+i2}/400"
            }

            override fun afterTextChanged(editable: Editable) {
                advice = repairIntroduceEditText.text.toString().trim { it <= ' ' }
//                setInformButtonState()
            }
        })

    }

    private fun setLocationRecyclerView() {
        setLocationData()
        repairLocationRecyclerView.layoutManager = GridLayoutManager(this, 4)
        bikeRepairAdapter = BikeRepairLocationAdapter(this, repairLocationList, R.layout.item_bike_repair_location)
        repairLocationRecyclerView.adapter = bikeRepairAdapter
        repairLocationRecyclerView.addItemDecoration(GridSpacingItemDecoration(4, DensityUtil.dip2px(this, 27f), false))

        bikeRepairAdapter?.let {
            it.setOnItemClickListner { _, position ->
                repairLocationList[position].isSelect = !repairLocationList[position].isSelect
                it.notifyDataSetChanged()

            }
        }


    }

    override fun onResume() {
        super.onResume()
        if (!ToolBox.isCompanyExclusive()) {
            presenter?.getAfterSale()
        }
    }

    /**
     * 设置损坏位置数据
     */
    private fun setLocationData() {
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_handlebar, "车把", false, 1))
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_light, "车灯", false, 2))
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_brake, "刹车", false, 3))
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_wheel, "车轮", false, 4))
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_battery, "电池", false, 5))
        repairLocationList.add(BikeRepairLocationResponse(R.mipmap.icon_bike_other, "其他", false, 6))
    }


     override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == TAKEPHOTO_CODE_INFORM) {
            if (resultCode == Activity.RESULT_OK) {
                val pathList = Album.parseResult(data)
                if (pathList == null && pathList.size == 0) {
                    return
                }
                if (isReTakePhoto) {
                    //本地图片列表删除
                    piclist.removeAt(position_retake)
                    //上传文件列表删除
                    fileList.removeAt(position_retake)
                    //替换为刚拍的照片    并压缩  添加到上传文件列表
                    piclist.add(position_retake, pathList[0])
                    compressImage(pathList[0])
                    fileList.add(position_retake, compressedImage)
                } else {
                    piclist.add(pathList[0])
                    compressImage(pathList[0])
                    fileList.add(compressedImage)
                }
                mPicShowAdapter?.notifyDataSetChanged()
//                setInformButtonState()

            } else if (resultCode == Activity.RESULT_CANCELED) {
                //取消
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data)
        }
    }


    //压缩图片
    private fun compressImage(picri: String) {
        val uri = Uri.parse("file://$picri")
        try {
            actualImage = CompressFileUtil.from(this, uri)
        } catch (e: IOException) {
            e.printStackTrace()
        }

        if (actualImage != null) {
            // Compress image in main thread using custom Compressor
            try {
                compressedImage = Compressor(this)
                        .setMaxWidth(640)
                        .setMaxHeight(480)
                        .setQuality(75)
                        .setCompressFormat(Bitmap.CompressFormat.JPEG)
                        .setDestinationDirectoryPath(Environment.getExternalStoragePublicDirectory(
                                Environment.DIRECTORY_PICTURES).absolutePath)
                        .compressToFile(actualImage)
                //                ImageLoader.loadSDCardPicture(ivCertificationPic,compressedImage.getAbsolutePath());


            } catch (e: IOException) {
                e.printStackTrace()
            }

        }
    }

    override fun bikeId(): Int = bikeId

    override fun description(): String = advice

    override fun getPic(): List<File?> = fileList

    override fun pmallOrderId(): Int = pmallOrderId

    override fun damagedPart(): String {
        var damagedPart = ""
        for (it in repairLocationList){
            if (it.isSelect){
                if (TextUtils.isEmpty(damagedPart)) {
                    damagedPart = "${it.introduce}"
                } else {
                    damagedPart = "$damagedPart、${it.introduce}"
                }
            }
        }
        return damagedPart
    }

    override fun showBuyLayout() {
        noService = true
        noneComboLayout.visibility = View.VISIBLE
        renewalLayout.visibility = View.GONE

    }

    override fun showRenewalLayout(buttonValue:String,placeholder:String,title:String) {
        noService = false
        noneComboLayout.visibility = View.GONE
        renewalLayout.visibility = View.VISIBLE
        renewalButton.text = buttonValue
        comboTimeTextView.text = placeholder
        comboTitle.text = title
    }

    override fun setPresenter(presenter: BikeRepairContract.Presenter?) {
    }



}