package com.baojia.mebike.feature.person_company.bike_repair

import android.app.Activity
import com.baojia.mebike.base.BasePresenter
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.data.response.UploadPhotoResponse
import com.baojia.mebike.data.response.company_personal.AfterSaleResponse
import com.baojia.mebike.data.response.repair.mainrepair.ReportPostResponse
import com.baojia.mebike.util.CollectionsWrapper
import com.baojia.mebike.util.Toast

class BikeRepairPresenter(activity: Activity, val mView: BikeRepairContract.View) : BasePresenter(activity), BikeRepairContract.Presenter {

    val model = BikeRepairModel(activity)
    private var imageUrlList = ""

    override fun postRepair() {
        addDisposable(model.postRepair(mView.bikeId(), mView.pmallOrderId(), mView.damagedPart(),
                mView.description(), imageUrlList, object : HttpCallback<ReportPostResponse>() {
            override fun onSucceed(data: ReportPostResponse) {
                super.onSucceed(data)
                data ?: return
                Toast.show(context, data.message)
                context.finish()

            }

            override fun onError(code: Int, message: String) {
                super.onError(code, message)
                onFailed(code, message)
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)
                Toast.show(context, message)
            }
        }))
    }

    override fun uploadPhoto() {
        if (CollectionsWrapper.isEmpty(mView.getPic())) {
            postRepair()
            return
        }
        addDisposable(model.uploadPhoto(mView.getPic(),
                object : HttpCallback<UploadPhotoResponse>() {
                    override fun onSucceed(data: UploadPhotoResponse?) {
                        super.onSucceed(data)
                        if (data == null || data.data == null) {
                            return
                        }
                        imageUrlList = data.data.imagesURL
                        postRepair()
                    }

                    override fun onFailed(code: Int, message: String) {
                        super.onFailed(code, message)
                        Toast.show(context, message)
                    }

                    override fun onError(code: Int, message: String) {
                        onFailed(-1, message)
                    }
                }))
    }

    override fun getAfterSale() {

        model?.getAfterSale(mView.bikeId(),object :HttpCallback<AfterSaleResponse.DataBean>(){
            override fun onSucceed(data: AfterSaleResponse.DataBean?) {
                super.onSucceed(data)
                if (data?.bought == 0) {
                    // 未购买
                    mView.showBuyLayout()
                } else if (data?.bought == 1) {
                    // 已购买
                    mView.showRenewalLayout(data.buttonText,data.placeholder,data.title)
                }
            }
        })
    }
}