package com.baojia.mebike.feature.person_company.usebike

import android.bluetooth.BluetoothGatt
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.provider.Settings
import android.text.TextUtils
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseActivity
import com.baojia.mebike.buletooth.*
import com.baojia.mebike.buletooth.BluetoothManager
import com.baojia.mebike.callback.IDialogCallback
import com.baojia.mebike.data.SubmitOrderData
import com.baojia.mebike.data.response.company_personal.PersonalBikeResponse
import com.baojia.mebike.data.response.company_personal.UseBikeResponse
import com.baojia.mebike.dialog.LeftRightButtonTipsDialog
import com.baojia.mebike.dialog.WaitDialog
import com.baojia.mebike.feature.exclusive.iWillExchange.IwillExchangeActivity
import com.baojia.mebike.feature.exclusive.shopping.orderdetails.ShoppingOrderDetailsActivity
import com.baojia.mebike.feature.exclusive.shopping.renewalfee.RenewalFeeAct
import com.baojia.mebike.feature.person_company.bike_location.BikeLocationActivity
import com.baojia.mebike.feature.person_company.bike_repair.BikeRepairActivity
import com.baojia.mebike.feature.usebike.dialog.BluetoothDialog
import com.baojia.mebike.feature.usebike.dialog.ReturnBikeDialog
import com.baojia.mebike.imageloader.ImageLoader
import com.baojia.mebike.util.CollectionsWrapper
import com.baojia.mebike.util.Toast
import com.baojia.mebike.util.ToolBox
import com.clj.fastble.BleManager
import com.clj.fastble.callback.BleGattCallback
import com.clj.fastble.callback.BleNotifyCallback
import com.clj.fastble.callback.BleScanCallback
import com.clj.fastble.callback.BleWriteCallback
import com.clj.fastble.data.BleDevice
import com.clj.fastble.exception.BleException
import com.clj.fastble.scan.BleScanRuleConfig
import kotlinx.android.synthetic.main.activity_use_bike_person.*

/**
 * +----------------------------------------------------------------------
 * | @time 2019/1/14:2:20 PM
 * | @author house
 * | @class_describe 正在用车页面
 * +----------------------------------------------------------------------
 **/
class UseBikeActivity : BaseActivity(), UseBikeContract.View {


    private val REQUEST_CODE_BLUETOOTH_ON_OFF_LINE = 0x123

    private var iBlueBoxContoller: IBlueBoxContoller? = null
    // 蓝牙执行结果
    private var bluetoothResult = -1

    private var waitDialog: WaitDialog? = null

    // 还车code值
    private var returnCode: Int = 0
    // 还车提示信息
    private var returnCarMessage: String? = null

    private val mHandler = Handler()

    companion object {
        fun start(mContext: Context, bikeId: Int = 0, orderNo: String = "") {
            val intent = Intent()
            intent.setClass(mContext, UseBikeActivity::class.java)
            intent.putExtra("bikeId", bikeId)
            intent.putExtra("orderNo", orderNo)
            mContext.startActivity(intent)
        }
    }

    private var bikeId: Int = 0
    private var mPresenter: UseBikeContract.Presenter? = null
    private var mapViewImp: UseBikeImp? = null
    private var menuItemAdapter: UseBikeMenuItemAdapter? = null
    private var menuItemData = arrayListOf<UseBikeResponse.DataBean.TagListBean>()
    private var changeBikeList = arrayListOf<PersonalBikeResponse.DataBean>()

    private var latitude: Double = 0.0
    private var longitude: Double = 0.0
    private var gpsTime: String? = ""

    private var orderNo: String = ""

    private var backBikeDialog: ReturnBikeDialog? = null

    override fun layoutId(): Int {
        return R.layout.activity_use_bike_person
    }

    override fun bindView(savedInstanceState: Bundle?) {

        mapView.onCreate(savedInstanceState)

        SubmitOrderData.isAppointment = false

        bikeId = intent.getIntExtra("bikeId", 0)
        orderNo = intent.getStringExtra("orderNo")

        mPresenter = UseBikePresenter(this, this)
        mapViewImp = UseBikeImp(mapView)

        setSupportActionBar(useBikeToolbar)
        supportActionBar?.setDisplayShowTitleEnabled(false)
        setClickListener()

        if (ToolBox.isCompanyExclusive()) {
            setCompanyView()
        } else {
            setPersonView()
            useBikeToolbar.setNavigationIcon(R.mipmap.back_icon)
        }

        setMenuItemRecyclerView()

        mPresenter?.getBikeBoxInfo()

        useBikeRightButton.visibility = View.GONE


        BleManager.getInstance().init(application)
        BleManager.getInstance()
                .enableLog(true)
                .setReConnectCount(3, 3000)
                .setConnectOverTime(3000).operateTimeout = 10000
    }


    private fun setMenuItemRecyclerView() {
        menuItemAdapter = UseBikeMenuItemAdapter(this, menuItemData, R.layout.item_use_bike_person)
        itemRecyclerView.layoutManager = LinearLayoutManager(this)
        itemRecyclerView.adapter = menuItemAdapter

        menuItemAdapter?.setOnItemClickListner { view, position ->

            if (view == null) {
                when (position) {
                    10 -> // 车辆续期
                        RenewalFeeAct.start(this@UseBikeActivity, 0, bikeId, mPresenter?.pmallOrderId(), mPresenter?.virGroupId()?.toInt())
                    11 -> // 电池续期
                        RenewalFeeAct.start(this@UseBikeActivity, 1, bikeId, mPresenter?.pmallOrderId(), mPresenter?.virGroupId()?.toInt())
                    else -> {
                        // 查看电池续期订单详情
                        mPresenter?.batteryProductOrderNo()?.let {
                            ShoppingOrderDetailsActivity.start(this@UseBikeActivity, it,
                                    mPresenter?.plateNo())
                        }
                    }
                }
                return@setOnItemClickListner
            }

            when (menuItemData[position].tag) {
                2 -> {
                    // 换电
                    IwillExchangeActivity.start(this@UseBikeActivity, bikeId, mPresenter?.changeBatteryOrderNo(), mPresenter?.productId()!!, mPresenter?.productOrderNo(), mPresenter?.virGroupId(), mPresenter?.pmallOrderId())
                }
                3 -> {
                    // 维修保养
                    BikeRepairActivity.start(this@UseBikeActivity, mPresenter?.pmallOrderId(), bikeId, mPresenter?.virGroupId())
                }
                4 -> {
                    // 归还车辆
                    val dialog = LeftRightButtonTipsDialog.show(supportFragmentManager, null, "确认要归还车辆吗？", 2, R.mipmap.yellow_tips_icon, "再想想")

                    dialog.setDialogCallback(object : IDialogCallback() {
                        override fun onConfirm() {
                            super.onConfirm()
                            mPresenter?.returnBike()
                        }

                    })
                }
            }
        }
    }

    private fun setClickListener() {
        setViewClickDuration(useBikeRightButton, 1)
        setViewClickDuration(whistleButton, 1)
        setViewClickDuration(openLockButton, 1)
        setViewClickDuration(closeLockButton, 1)
        setViewClickDuration(mapViewBg, 1)
        setViewClickDuration(changeBikeButton, 1)
        useBikeToolbar.setNavigationOnClickListener {
            finish()
        }
    }

    /**
     * 企业专享显示
     */
    private fun setCompanyView() {
        useBikeTitleTextView.text = "正在用车"
        companyLogoImageView.visibility = View.VISIBLE
    }

    /**
     * 个人专享显示
     */
    private fun setPersonView() {
        useBikeTitleTextView.text = "我的专享"
        bikeNameTextView.visibility = View.VISIBLE
    }

    override fun onResume() {
        super.onResume()
        mPresenter?.getBikeInfo()
        mPresenter?.getBikeBoxInfo()
        mapViewImp?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapViewImp?.onPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        mapViewImp?.onDestroy()
        BleManager.getInstance().destroy()
    }


    override fun setChangeBikeButtonVisibility(isShow: Boolean) {
        if (isShow) {
            changeBikeButton.visibility = View.VISIBLE
        } else {
            changeBikeButton.visibility = View.GONE
        }
    }

    override fun setViewClick(view: View?) {
        super.setViewClick(view)
        when (view) {
            useBikeRightButton -> {
                // 使用说明
                mPresenter?.useIntroduceClickListener()
            }
            whistleButton -> {
                // 鸣笛
                mPresenter?.sendBoxInstruct("C")
            }
            openLockButton -> {
                // 开锁
                operation = "A"
                startCarByBluetooth()
//                mPresenter?.sendBoxInstruct("A")
            }
            closeLockButton -> {
                // 关锁
                operation = "B"
                stopCarByBluetooth(false)
//                mPresenter?.sendBoxInstruct("B")
            }
            mapViewBg -> {
                if (longitude > 0 || latitude > 0) {
                    // 跳转地图
                    BikeLocationActivity.start(this@UseBikeActivity, bikeId)
                }
            }
            changeBikeButton -> {
                // 切换车辆
                if (!CollectionsWrapper.isEmpty(changeBikeList)) {
                    showChangeBikeDialog()
                } else {
                    mPresenter?.getBikeList()
                }
            }
        }
    }

    private fun showChangeBikeDialog() {
        var changeBikeDialog = ChangeBikeDialog.show(supportFragmentManager, changeBikeList)
        changeBikeDialog.setDialogCallback(object : IDialogCallback() {
            override fun onConfirm(content: String) {
                super.onConfirm(content)
                val position = content.toInt()
                bikeId = changeBikeList[position].bikeId
                mPresenter?.getBikeBoxInfo()
                mPresenter?.getBikeInfo()
            }
        })
    }

    override fun bikeId(): Int = bikeId

    override fun setBatteryInfo(batteryLevel: Int, batteryLevelText: String, extensionMileage: Int, extensionMileageText: String) {
        canDriveDistanceTextView.text = extensionMileageText
        canDriveDistanceProgress.progress = batteryLevel
        batteryTextView.text = batteryLevelText
        batteryProgress.progress = batteryLevel
    }

    override fun setBikeImage(bikeUrl: String) {
        ImageLoader.loadUrl(bikeImageView, bikeUrl)
    }

    override fun setBikeName(bikeName: String) {
        bikeNameTextView.text = bikeName
    }


    override fun setBikeLocation(latitude: Double, longitude: Double, address: String?, gpsTime: String?) {
        mapViewImp?.addBikeMarker(latitude, longitude)
        bikeAddressTextView.text = address
        this.gpsTime = gpsTime
        this.latitude = latitude
        this.longitude = longitude
    }

    override fun setMenuItemData(itemData: ArrayList<UseBikeResponse.DataBean.TagListBean>) {
        menuItemData.clear()
        menuItemData.addAll(itemData)
        menuItemAdapter?.notifyDataSetChanged()
    }

    override fun setChangeBikeListData(mData: ArrayList<PersonalBikeResponse.DataBean>) {
        changeBikeList.clear()
        changeBikeList.addAll(mData)
        showChangeBikeDialog()
    }

    override fun showBackBikeDialog() {
        if (backBikeDialog != null && backBikeDialog!!.isShowing) {
            return
        }
        backBikeDialog = ReturnBikeDialog(this)
        backBikeDialog?.let {
            it.show()
            it.setDialogConfirmClickCallback {
                mPresenter?.unSubscribe()
            }
        }
    }

    override fun dismissBackBikeDialog() {
        backBikeDialog?.let {
            it.dismiss()
            null
        }
    }

    override fun orderNo(): String = orderNo

    override fun hintOpenBluetooth() {
    }

    override fun stopCarByBluetooth(isReturn: Boolean, code: Int, message: String?) {
        this.isReturnCar = isReturn
        this.returnCode = code
        this.returnCarMessage = message
        stopCarByBluetooth(isReturnCar)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CODE_BLUETOOTH_ON_OFF_LINE) {
            if (BluetoothManager.isBluetoothEnabled()) {
                // 蓝牙是否打开
                // 关闭开启蓝牙提示
                // 提示蓝牙已开启，连接蓝牙，
//                openBluetoothHintLayout.setVisibility(View.GONE)
                Toast.show(this, "蓝牙已开启，可以使用蓝牙操作车辆啦")
            } else {
            }
        }
    }

    private var cmd1: ByteArray? = null
    // B 锁车 A 启动
    private var operation: String = ""

    private fun showLoading() {
        if (waitDialog == null) {
            waitDialog = WaitDialog(this)
            waitDialog!!.setCancelable(false)
            waitDialog!!.setCanceledOnTouchOutside(false)
        }
        if (!waitDialog!!.isShowing) {
            waitDialog!!.show()
        }
    }

    private fun dismissLoading() {
        waitDialog?.let {
            it.dismiss()
        }
    }

    // 是否还车
    private var isReturnCar = false

    /**
     * 弹出系统弹框提示用户打开 BluetoothActivity
     */
    private fun turnOnBluetooth() {
        val bluetoothDialog = BluetoothDialog()
        bluetoothDialog.show(supportFragmentManager, "BluetoothDialog")
        bluetoothDialog.setBluetoothCallback(object : BluetoothDialog.BluetoothCallback {
            override fun onConfirm() {
                val requestBluetoothOn = Intent(Settings.ACTION_BLUETOOTH_SETTINGS)

                // 请求开启 BluetoothActivity
                startActivityForResult(requestBluetoothOn,
                        REQUEST_CODE_BLUETOOTH_ON_OFF_LINE)
                bluetoothDialog.dismiss()
            }

            override fun onDismiss() {
                bluetoothDialog.dismiss()
                mPresenter?.sendBoxInstruct(operation)
            }

            override fun onCancel() {
                // 点击返回键或者点击空白关闭
                mPresenter?.sendBoxInstruct(operation)
            }
        })

    }

    /**
     * 设置蓝牙发送指令失败判断逻辑
     */
    private fun setBluetoothSendFail() {
        if (isReturnCar) {
            mPresenter?.sendBoxInstruct("B", true, null)
        } else {
            mPresenter?.sendBoxInstruct(operation, false, null)
        }
    }

    /**
     * 锁车
     *
     * @param isReturnCar
     */
    private fun stopCarByBluetooth(isReturnCar: Boolean) {
        if (isCanConnectBle()) {
            bluetoothResult = -1
            operation = "B"
//            mPresenter.saveBluetoothLog("N")
            getCmd(2)
            //            blueOperation(cmd1);
            ////            write(bluetoothLeDevice, cmd1);
            if (!isReturnCar) {
                showLoading()
            }
            //            setBluetoothConnectTime(isReturnCar);
            if (isConnect()) {
                write()
            } else {
                startScan()
            }
        } else {
            if (bikeId == 0) {
                return
            }
            mPresenter?.sendBoxInstruct(operation, isReturnCar, null)
        }

    }

    /**
     * 蓝牙启动
     */
    private fun startCarByBluetooth() {
        if (isCanConnectBle()) {
            bluetoothResult = -1
            operation = "A"
            getCmd(1)
            showLoading()
            if (isConnect()) {
                write()
            } else {
                startScan()
            }
        } else {
            if (bikeId == 0) {
                return
            }
            mPresenter?.sendBoxInstruct(operation, isReturnCar, null)
        }
    }

    /**
     * 根据不同的车辆类型获取蓝牙指令
     *
     * @param type type 1:开锁 2：锁车
     */
    private fun getCmd(type: Int) {
        if (mPresenter?.boxType() == 1) {
            // 小蜜蜂
            cmd1 = BluetoothCmdUtils.xiaomiCmd(type)
        } else if (mPresenter?.boxType() == 2) {
            // 蜜蜂
            cmd1 = BluetoothCmdUtils.mifengCmd(type)
        }
    }


    private var bikeBleDevice: BleDevice? = null

    /**
     * 扫描
     */
    private fun startScan() {
        BleManager.getInstance().scan(object : BleScanCallback() {
            override fun onScanStarted(success: Boolean) {

            }

            override fun onScanning(bleDevice: BleDevice?) {

                if (bleDevice != null) {
                    if (TextUtils.equals(bleDevice.mac, mPresenter?.macAddress())) {
                        // 扫描到了，开始连接
                        BleManager.getInstance().cancelScan()
                        bikeBleDevice = bleDevice
                        connect(bleDevice)
                    }
                }
            }

            override fun onScanFinished(scanResultList: List<BleDevice>) {
                if (CollectionsWrapper.isEmpty(scanResultList)) {
                    dismissLoading()
                    mPresenter?.sendBoxInstruct(operation, false, null)
                }
            }
        })
    }

    private fun connect(bleDevice: BleDevice?) {
        BleManager.getInstance().connect(bleDevice, object : BleGattCallback() {

            override fun onStartConnect() {
                // 开始连接
            }

            override fun onConnectFail(bleDevice: BleDevice, exception: BleException) {
                // 连接失败
                if (waitDialog != null && waitDialog!!.isShowing) {
                    BleManager.getInstance().destroy()
                    dismissLoading()
                    setBluetoothSendFail()
                }
            }

            override fun onConnectSuccess(bleDevice: BleDevice, gatt: BluetoothGatt, status: Int) {
                // 连接成功

                setNotify()

                write()
            }

            override fun onDisConnected(isActiveDisConnected: Boolean, device: BleDevice, gatt: BluetoothGatt, status: Int) {
                // 断开连接
                if (isReturnCar) {
                    if (backBikeDialog != null && backBikeDialog!!.isShowing) {
                        mPresenter?.sendBoxInstruct( "B", true,null)
                    }
                } else {
                    if (waitDialog != null && waitDialog!!.isShowing) {
                        dismissLoading()
                        mPresenter?.sendBoxInstruct( operation, false,null)
                    }
                }

            }
        })
    }


    private fun setNotify() {
        BleManager.getInstance().notify(bikeBleDevice, getBluetoothConfig()!!.uuidService.toString(), getBluetoothConfig()!!.uuidNotifyRead.toString(), object : BleNotifyCallback() {
            override fun onNotifySuccess() {
                println("setNotify成功====")
                write()
            }

            override fun onNotifyFailure(exception: BleException) {
                println("setNotify失败====")
                //                BleManager.getInstance().destroy();
            }

            override fun onCharacteristicChanged(data: ByteArray?) {
                dismissLoading()
                if (data == null) {
                    return
                }
                // 小蜜蜂数据解析
                if (mPresenter?.boxType() == 1) {
                    var length: Byte = 0
                    if (data!!.size > 1) {
                        length = data[1]
                    }
                    if (data.size > 1 + length) {
                        if (data[1 + length].toInt() == 0) {
                            bluetoothResult = 1
//                        mPresenter.saveBluetoothLog()
                            if (TextUtils.equals(operation, "B")) {
                                if (isReturnCar) {
                                    mPresenter?.setCode(returnCode, returnCarMessage, true)
                                    return
                                }
                                Toast.show(this@UseBikeActivity, "蓝牙锁车成功")
                            } else {
                                Toast.show(this@UseBikeActivity, "蓝牙启动成功")
                            }
                        } else {
                            bluetoothResult = 0
//                        mPresenter.saveBluetoothLog()
                            if (TextUtils.equals(operation, "B")) {
                                Toast.show(this@UseBikeActivity, "蓝牙锁车失败")
                            } else {
                                Toast.show(this@UseBikeActivity, "蓝牙启动失败")
                            }
                            setBluetoothSendFail()
                        }
                    }
                } else if (mPresenter?.boxType() == 2) {
                    // 蜜蜂数据解析
                    if (data != null && data.size > 7) {
                        if (data[0] == 0x4d.toByte()) {
                            when (data[1]) {
                                0x01.toByte() -> {
                                    //开锁 data[6]
                                    if (data[6] == 0x00.toByte()) {
                                        Toast.show(this@UseBikeActivity, "蓝牙开锁成功")
                                        bluetoothResult = 1
                                    } else {
                                        bluetoothResult = 0
                                        Toast.show(this@UseBikeActivity, "蓝牙启动失败")
                                        setBluetoothSendFail()
                                    }
//                                mPresenter.saveBluetoothLog()
                                }
                                0x02.toByte() -> if (data[6] == 0x00.toByte()) {
                                    bluetoothResult = 1
//                                mPresenter.saveBluetoothLog()
                                    if (isReturnCar) {
                                        mPresenter?.setCode(returnCode, returnCarMessage, true)
                                        return
                                    }
                                    Toast.show(this@UseBikeActivity, "蓝牙锁车成功")
                                } else {
                                    bluetoothResult = 0
//                                mPresenter.saveBluetoothLog()
                                    Toast.show(this@UseBikeActivity, "蓝牙锁车失败")
                                    setBluetoothSendFail()
                                }
                                0x03.toByte() -> {
                                }
                            }
                        }
                    }
                }
            }
        })

    }

    private fun getBluetoothConfig(): IBluetoothConfig? {
        var bluetoothConfig: IBluetoothConfig? = null
        if (mPresenter?.boxType() == 1) {
            // 小蜜蜂
            bluetoothConfig = XiaoMiConfig()
        } else if (mPresenter?.boxType() == 2) {
            // 蜜蜂
            bluetoothConfig = MiFengConfig()
        }
        return bluetoothConfig
    }

    private fun write() {
//        showLoading()

        BleManager.getInstance().write(bikeBleDevice, getBluetoothConfig()!!.uuidService.toString(), getBluetoothConfig()!!.uuidNotify.toString(),
                cmd1, object : BleWriteCallback() {
            override fun onWriteSuccess(current: Int, total: Int, data: ByteArray) {
                println("执行成功====")

                dismissLoading()
            }

            override fun onWriteFailure(exception: BleException) {
                dismissLoading()
            }
        })
    }

    /**
     * 是否可以使用蓝牙
     * @return
     */
    private fun isCanConnectBle(): Boolean {
        return !TextUtils.isEmpty(mPresenter?.macAddress()) && BluetoothManager.isBluetoothSupported() && BluetoothManager.isBluetoothEnabled() && (mPresenter?.boxType() == 1 || mPresenter?.boxType() == 2)
    }

    /**
     * 蓝牙是否已连接
     * @return
     */
    private fun isConnect(): Boolean {
        return bikeBleDevice != null && BleManager.getInstance().isConnected(bikeBleDevice)
    }

    override fun setLogoImage(logoImageUrl: String) {
        ImageLoader.loadUrl(companyLogoImageView, logoImageUrl)
    }

    override fun setRightButtonVisible(isShow: Boolean) {
        useBikeRightButton.visibility = if (isShow) View.VISIBLE else View.GONE
    }

    override fun setBikeId(bikeId: Int) {
        this.bikeId = bikeId
    }


    override fun setPresenter(presenter: UseBikeContract.Presenter?) {
        setIPresenter(presenter)
    }

    override fun onBackPressed() {
        if (!ToolBox.isCompanyExclusive()) {
            super.onBackPressed()
        }
    }

    override fun setScanRule() {
        val scanRuleConfig = BleScanRuleConfig.Builder()
                .setDeviceMac(mPresenter?.macAddress())           // 只扫描指定mac的设备，可选
                //            .setServiceUuids(serviceUuids)      // 只扫描指定的服务的设备，可选
                //            .setDeviceName(true, names)   // 只扫描指定广播名的设备，可选
                //            .setAutoConnect(isAutoConnect)      // 连接时的autoConnect参数，可选，默认false
                .setScanTimeOut(3000)              // 扫描超时时间，可选，默认10秒
                .build()
        BleManager.getInstance().initScanRule(scanRuleConfig)
    }
}