package com.baojia.mebike.feature.person_company.usebike

import com.baojia.mebike.base.IBasePresenter
import com.baojia.mebike.base.IBaseView
import com.baojia.mebike.data.response.company_personal.PersonalBikeResponse
import com.baojia.mebike.data.response.company_personal.UseBikeResponse

/**
 * +----------------------------------------------------------------------
 * | @time 2019/1/14:5:17 PM
 * | @author house
 * | @class_describe
 * +----------------------------------------------------------------------
 **/
interface UseBikeContract {

    interface View : IBaseView<Presenter> {
        /**
         * 设置是否显示切换按钮
         */
        fun setChangeBikeButtonVisibility(isShow: Boolean)

        /**
         * 车辆id
         */
        fun bikeId(): Int

        /**
         * 设置电量信息
         */
        fun setBatteryInfo(batteryLevel: Int, batteryLevelText: String, extensionMileage: Int, extensionMileageText: String)

        /**
         * 设置车辆图片
         */
        fun setBikeImage(bikeUrl: String)

        /**
         * 设置车辆名
         */
        fun setBikeName(bikeName: String)

        /**
         * 设置车辆位置
         */
        fun setBikeLocation(latitude: Double, longitude: Double, address: String?, gpsTime: String?)

        fun setMenuItemData(itemData: ArrayList<UseBikeResponse.DataBean.TagListBean>)

        fun setChangeBikeListData(mData: ArrayList<PersonalBikeResponse.DataBean>)

        /**
         * 显示还车dialog
         */
        fun showBackBikeDialog()

        /**
         * 取消还车dialog
         */
        fun dismissBackBikeDialog()

        /**
         * 订单号
         */
        fun orderNo(): String

        /**
         * 提示蓝牙操作
         */
        fun hintOpenBluetooth()

        /**
         * 蓝牙锁车
         *
         * @param isReturn
         * @param code
         * @param message
         */
        fun stopCarByBluetooth(isReturn: Boolean, code: Int, message: String?)

        /**
         * 是否显示使用说明
         */
        fun setRightButtonVisible(isShow:Boolean)

        fun setLogoImage(logoImageUrl:String)

        fun setBikeId(bikeId:Int)

        fun setScanRule()
    }

    interface Presenter : IBasePresenter {

        fun getBikeInfo()
        /**
         * 发送指令
         */
        fun sendBoxInstruct(operation: String, isReturnBike: Boolean = false, key: String? = null)

        /**
         * 换电订单号
         */
        fun changeBatteryOrderNo(): String?

        fun getBikeList()
        /**
         * 获取车辆盒子信息
         */
        fun getBikeBoxInfo()

        /**
         * 盒子类型
         */
        fun boxType(): Int

        /**
         * mac地址
         */
        fun macAddress(): String?

        /**
         * 知否支持蓝牙操作
         */
        fun isBluetoothEnableByBox(): Boolean

        /**
         * 还车
         */
        fun returnBike()

        fun setCode(returnCarCode: Int, returnCarMessage: String?, isReturnCar: Boolean)

        /**
         * 专属车订单id
         */
        fun pmallOrderId() : Int

        /**
         * 虚拟组id
         */
        fun virGroupId():Long

        fun productId():Long

        fun productOrderNo():String?
        /**
         * 使用说明按钮点击事件
         */
        fun useIntroduceClickListener()

        /**
         * 电池续期订单号
         */
        fun batteryProductOrderNo():String?

        /**
         * 车牌号
         */
        fun plateNo():String
    }
}