package com.baojia.mebike.feature.person_company.usebike

import android.app.Activity
import android.content.Intent
import android.os.Handler
import android.text.TextUtils
import com.baojia.mebike.base.BasePresenter
import com.baojia.mebike.buletooth.BluetoothManager
import com.baojia.mebike.callback.HttpCallback
import com.baojia.mebike.data.response.BaseResponse
import com.baojia.mebike.data.response.bike.BoxInfoResponse
import com.baojia.mebike.data.response.bike.SendBoxInstructResponse
import com.baojia.mebike.data.response.company_personal.PersonalBikeResponse
import com.baojia.mebike.data.response.company_personal.UseBikeResponse
import com.baojia.mebike.feature.pay.succeed.CompanyFinishActivity
import com.baojia.mebike.util.CollectionsWrapper
import com.baojia.mebike.util.JumpActivity
import com.baojia.mebike.util.Toast
import com.baojia.mebike.util.ToolBox
import io.reactivex.disposables.Disposable

/**
 * +----------------------------------------------------------------------
 * | @time 2019/1/14:5:31 PM
 * | @author house
 * | @class_describe
 * +----------------------------------------------------------------------
 **/
class UseBikePresenter(var mContext: Activity, var mView: UseBikeContract.View) : BasePresenter(mContext), UseBikeContract.Presenter {


    private var useBikeModel: UseBikeModel? = null
    private var disposable: Disposable? = null
    private var changeBatteryOrderNo: String? = null
    private var pmallOrderId: Int = 0
    private var productOrderNo: String? = null

    // 1-小安盒子 2-南方数码微电云盒子
    private var boxType: Int = 0
    // 蓝牙地址
    private var macAdress: String? = null

    private var productId: Long = 0
    private var plateNo: String = ""
    private var batteryProductOrderNo: String? = ""

    private var isBluetoothEnableByBox = false

    private var useIntroduceUrl: String? = null
    // 虚拟组id
    private var virGroupId:Long = 0

    init {
        useBikeModel = UseBikeModel(mContext)
    }

    override fun getBikeInfo() {

        removeDisposable(disposable)

        addDisposable(useBikeModel?.getBikeInfo(mView.bikeId(), object : HttpCallback<UseBikeResponse.DataBean>() {
            override fun onSucceed(data: UseBikeResponse.DataBean) {
                super.onSucceed(data)

                mView.setBatteryInfo(data.batteryLevel, data.batteryLevelText, data.extensionMileage, data.extensionMileageText)
                if (data.location != null && data.location.size == 2) {
                    mView.setBikeLocation(data.location[1], data.location[0], data.address, data.gpsTime)
                }
                if (!ToolBox.isCompanyExclusive()) {
                    mView.setBikeName(data.name)
                    mView.setChangeBikeButtonVisibility(data.switchFlag == 1)
                }
                mView.setBikeImage(data.imgUrl)

                if (!CollectionsWrapper.isEmpty(data.tagList)) {
                    mView.setMenuItemData(data.tagList)
                }
                virGroupId = data.virGroupId
                mView.setBikeId(data.bikeId)
                changeBatteryOrderNo = data.changeOrderNo
                pmallOrderId = data.pmallOrderId
                productId = data.productId
                productOrderNo = data.orderNo
                useIntroduceUrl = data.instructionsUrl
                plateNo = data.plateNo
                batteryProductOrderNo = data.lastChangeOrderNo
                mView.setRightButtonVisible(!TextUtils.isEmpty(useIntroduceUrl))
                if (!TextUtils.isEmpty(data.logoImgUrl)) {
                    mView.setLogoImage(data.logoImgUrl)
                }
            }
        }))
    }

    override fun sendBoxInstruct(operation: String, isReturnBike: Boolean, key: String?) {

        var isShowLoading = false
        if ((TextUtils.equals(operation, "B") || TextUtils.equals(operation, "A") || TextUtils.equals(operation, "C")) && !isReturnBike) {
            isShowLoading = true
        }
        if (isReturnBike) {
            mView.showBackBikeDialog()
            if (TextUtils.equals(operation, "B")) {
                // 锁车
                currentSendCount++
            }
        }

        useBikeModel?.sendBoxInstruct(operation, mView.bikeId(), null, null, isShowLoading, object : HttpCallback<SendBoxInstructResponse>() {
            override fun onSucceed(data: SendBoxInstructResponse?) {
                super.onSucceed(data)

                if (!isReturnBike) {
                    Toast.show(context, data?.message)
                    sendBoxInstructFail()
                    return
                }

                if (data?.data == null) {
                    if (isReturnBike) {
                        if (currentSendCount >= MAX_SEND_COUNT) {
                            // 开始还车
                            mView.showBackBikeDialog()
                            currentSendCount = 0
                            setCode(returnCarCode, returnCarMessage, true)
                            return
                        }
                        // 锁车
                        sendBoxInstruct("B", true, null)
                    }
                    return
                }

                if (!isReturnBike) {
                    return
                }

                if (TextUtils.equals(operation, "B")) {
                    Handler().postDelayed({
                        // 查询
                        sendBoxInstruct("Q", true, data.data.msgKey)
                    }, 1500)

                } else {
                    // 开始还车
                    mView.showBackBikeDialog()
                    currentSendCount = 0
                    setCode(returnCarCode, returnCarMessage, true)
                }
            }

            override fun onFailed(code: Int, message: String) {
                super.onFailed(code, message)

                if (isReturnBike) {
                    if (currentSendCount >= MAX_SEND_COUNT) {
                        // 开始还车
                        mView.showBackBikeDialog()
                        currentSendCount = 0
                        setCode(returnCarCode, returnCarMessage, true)
                        return
                    }
                    // 锁车
                    sendBoxInstruct("B", true, null)
                    return
                } else {
                    // 提示蓝牙操作
                    Toast.show(context, message)
                    sendBoxInstructFail()
                    return
                }
            }

            override fun onError(code: Int, message: String) {
                super.onError(code, message)
                onFailed(code, message)
            }
        })

    }

    /**
     * 通用code值处理
     *
     * @param returnCarCode
     * @param returnCarMessage
     * @param isReturnCar
     */
    override fun setCode(returnCarCode: Int, returnCarMessage: String?, isReturnCar: Boolean) {
        mView.dismissBackBikeDialog()
        when (returnCarCode) {
            400 -> {
                unSubscribe()
                // 跳转支付页面
                JumpActivity.JumpPaymentActivity(context, mView.orderNo())
                Toast.show(context, returnCarMessage)
                context.finish()
                return
            }
            405 -> {
                unSubscribe()
                // 支付完成
                if (ToolBox.isCompanyExclusive()) {
                    context.startActivity(Intent(context, CompanyFinishActivity::class.java))
                    context.finish()
                    return
                }
                JumpActivity.JumpPayFinishActivity(context, mView.orderNo())
                context.finish()

                return
            }
            401 -> {
                unSubscribe()
                // 订单取消
                return
            }
            403 -> {
                unSubscribe()
                // 订单取消
            }
            200 -> {
            }
            else -> Toast.show(context, returnCarMessage)
        }
        if (isReturnCar) {
            if (returnCarCode == 200) {
                // 支付完成
                if (ToolBox.isCompanyExclusive()) {
                    context.startActivity(Intent(context, CompanyFinishActivity::class.java))
                    context.finish()
                    return
                }
                // 跳转支付
                JumpActivity.JumpPaymentActivity(context, mView.orderNo())
                context.finish()
                return
            }
        }
    }

    /**
     * 发送指令失败
     */
    private fun sendBoxInstructFail() {
        if (!isBluetoothEnableByBox) {
            return
        }
        if (!BluetoothManager.isBluetoothSupported()) {
            // 提示失败
            return
        }
        if (BluetoothManager.isBluetoothEnabled()) {
            return
        }
        // 提示打开蓝牙
        mView.hintOpenBluetooth()
    }

    override fun changeBatteryOrderNo(): String? = changeBatteryOrderNo

    override fun getBikeList() {
        useBikeModel?.getBikeList(object : HttpCallback<ArrayList<PersonalBikeResponse.DataBean>>() {
            override fun onSucceed(data: ArrayList<PersonalBikeResponse.DataBean>) {
                super.onSucceed(data)
                mView?.setChangeBikeListData(data)
            }
        })
    }

    override fun getBikeBoxInfo() {
        useBikeModel?.getBoxInfo(mView.bikeId(), object : HttpCallback<BoxInfoResponse>() {
            override fun onSucceed(data: BoxInfoResponse) {
                super.onSucceed(data)
                if (data.data != null) {
                    boxType = data.data.boxType
                    macAdress = data.data.btMac
                    isBluetoothEnableByBox = !TextUtils.isEmpty(macAdress)
                    if (isBluetoothEnableByBox) {
                        mView.setScanRule()
                    }
                }
            }
        })
    }

    private var currentSendCount = 0
    // 盒子指令最多发送次数
    private val MAX_SEND_COUNT = 3
    // 还车接口返回的code值
    private var returnCarCode: Int = 0
    private var returnCarMessage: String? = null

    override fun returnBike() {
        mView.showBackBikeDialog()
        useBikeModel?.returnBike(mView.orderNo(), object : HttpCallback<BaseResponse>() {
            override fun onSucceed(data: BaseResponse?) {
                super.onSucceed(data)
                onFailed(data?.code!!, data?.message)
            }

            override fun onFailed(code: Int, message: String?) {
                super.onFailed(code, message)
                if (code == 200 || code == 401 || code == 403 || code == 405) {
                    currentSendCount = 0
                    returnCarCode = code
                    returnCarMessage = message

                    if (isBluetoothEnableByBox && BluetoothManager.isBluetoothSupported()
                            && BluetoothManager.isBluetoothEnabled()) {
                        mView.stopCarByBluetooth(true, code, returnCarMessage)
                    } else {
                        sendBoxInstruct("B", true, null)
                    }
                    return
                }
                Toast.show(context, message)
                mView.dismissBackBikeDialog()
            }

            override fun onError(code: Int, message: String?) {
                super.onError(code, message)
                onFailed(code, message)
            }
        })
    }

    override fun boxType(): Int = boxType

    override fun macAddress(): String? = macAdress

    override fun pmallOrderId(): Int = pmallOrderId

    override fun isBluetoothEnableByBox(): Boolean = isBluetoothEnableByBox

    override fun productId(): Long = productId

    override fun productOrderNo(): String? = productOrderNo

    override fun plateNo(): String = plateNo

    override fun batteryProductOrderNo(): String? = batteryProductOrderNo

    override fun virGroupId(): Long = virGroupId


    override fun useIntroduceClickListener() {
        if (!TextUtils.isEmpty(useIntroduceUrl)) {
            JumpActivity.JumpWebView(context, "使用说明", useIntroduceUrl)
        }
    }
}