package com.baojia.mebike.util.download

import android.annotation.TargetApi
import android.app.*
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.Handler
import android.text.method.ScrollingMovementMethod
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ProgressBar
import android.widget.TextView
import androidx.core.app.NotificationCompat
import androidx.core.content.FileProvider
import com.baojia.mebike.BuildConfig
import com.baojia.mebike.R
import com.baojia.mebike.base.BaseApplication
import com.baojia.mebike.data.cache.PreferencesUtil
import com.baojia.mebike.util.CommonUtils
import com.baojia.mebike.util.LogUtil
import com.baojia.mebike.util.Toast
import com.baojia.mebike.util.ToolBox
import com.liulishuo.okdownload.DownloadTask
import com.liulishuo.okdownload.SpeedCalculator
import com.liulishuo.okdownload.core.breakpoint.BlockInfo
import com.liulishuo.okdownload.core.breakpoint.BreakpointInfo
import com.liulishuo.okdownload.core.cause.EndCause
import com.liulishuo.okdownload.core.listener.DownloadListener4WithSpeed
import com.liulishuo.okdownload.core.listener.assist.Listener4SpeedAssistExtend
import java.io.File


/**
 * app更新管理
 */
open class ApkUpdateManager {



    var type = 1
    var fileName: String = ""
    var title: String = ""
    var desc: String = ""
    var url: String = ""
    var mContext: Context? = null
    // 下载百分比
    var progressTitleTypeTwo: TextView? = null
    /* 更新进度条 */
    var progressBarTypeTwo: ProgressBar? = null
    var task: DownloadTask? = null

    private var mNotificationManager: NotificationManager? = null
    private var mCBuilder: NotificationCompat.Builder? = null
    private var mNotification: Notification? = null
    private var mPendingIntent: PendingIntent? = null

    var repeatTime = 0
    //手动取消   防止错误重试一直进行
    var handCancle = false

    val NOTIFICATION_DOWNLOAD_CHANNEL_ID = "Downloading"
    val NOTIFICATION_DOWNLOAD_CHANNEL_NAME = ToolBox.getString(R.string.app_name)

    //开始下载任务
    /**
     * type 1:可选 2:强制 0:不更新
     * fileName  下载文件自定义名字(appname+version) 例如 mebike1.0.0
     * title   更新文案title
     * desc   更新文案描述
     * url   更新url
     * other   其他需求备用
     */
    fun doUpdate(mContext: Context, type: Int, fileName: String, title: String, desc: String, url: String) {
        this.type = type
        this.fileName = "$fileName.apk"
        this.title = title
        this.desc = desc
        this.url = url
        this.mContext = mContext
        when (type) {
            0 -> {
                //最新版本  不需要更新
            }

            1 -> {
                //选择性更新
                showUpdateDialog()
            }
            2 -> {
                //强制更新
                showUpdateDialog()
            }

        }
    }


    /**
     * 显示软件更新对话框 自定义样式
     */
    private fun showUpdateDialog() {

        val dialog = Dialog(mContext, R.style.waitDialog)
        val view = ToolBox.getLayoutInflater(mContext).inflate(R.layout.update_dialog, null)
        val params = ViewGroup.LayoutParams((ToolBox.getDisplayMetrics().widthPixels * 0.8).toInt(), ViewGroup.LayoutParams.WRAP_CONTENT)
        dialog.setContentView(view, params)
        if (type == 2) {
            dialog.setCancelable(false)
        } else {
            dialog.setCancelable(true)
            dialog.setCanceledOnTouchOutside(true)
        }
        dialog.show()
        val tipTitle = view.findViewById<TextView>(R.id.update_dialog_title)
        val tipTextView = view.findViewById<TextView>(R.id.update_dialog_tips)
        val updateDialogCancel = view.findViewById<Button>(R.id.update_dialog_cancel)
        val btn_update = view.findViewById<Button>(R.id.update_dialog_sure)
        tipTextView.movementMethod = ScrollingMovementMethod.getInstance()
        tipTitle.text = title// 设置加载信息
        tipTextView.text = desc// 设置加载信息

        if (type == 2) {
            updateDialogCancel.visibility = View.GONE
        } else {
            updateDialogCancel.setOnClickListener { dialog.dismiss() }
        }


        btn_update.setOnClickListener {
            if (type == 2) {
                // 显示下载对话框   强制更新  只显示进度条   不可取消
                showDownloadDialog()
                handCancle = false
            } else {
                // 通知栏提示
                if (CommonUtils.isNotificationEnabled(mContext)) {
                    createNotification()
                    downloadApk()
                    handCancle = false
                    repeatTime = 0
                    Toast.show(BaseApplication.getInstance(), "已在后台下载")
                    dialog.dismiss()
                } else {
                    Toast.show(BaseApplication.getInstance(), "请先打开通知栏权限")
                    CommonUtils.toSetting(mContext)
                }
            }

        }


    }

    /**
     * 强制下载进度框
     */
    private fun showDownloadDialog() {
        var mDownloadDialog = Dialog(mContext, R.style.waitDialog)
        mDownloadDialog.setContentView(R.layout.wenxin_prompt)
        mDownloadDialog.setCancelable(false)
        mDownloadDialog.show()
        progressTitleTypeTwo = mDownloadDialog.findViewById(R.id.progress_title) as TextView
        progressBarTypeTwo = mDownloadDialog.findViewById(R.id.update_progress) as ProgressBar
        val cancelbtn = mDownloadDialog.findViewById(R.id.promptBtn) as Button
        cancelbtn.setOnClickListener {
            mDownloadDialog.dismiss()
            cancleTask()
            handCancle = true
        }
        // 下载文件
        downloadApk()
    }

    /**
     * 开启下载任务
     */
    private fun downloadApk() {
        mContext ?: return
        val parentFile = getParentFile()
        task = DownloadTask.Builder(url, parentFile)
                .setFilename(fileName)
                // the minimal interval millisecond for callback progress
                .setMinIntervalMillisCallbackProcess(16)
                // ignore the same task has already completed in the past.
                .setPassIfAlreadyCompleted(false)
                .build()

        task?.enqueue(object : DownloadListener4WithSpeed() {

            var totalLength: Long = -1
            var time = 0L

            override fun taskStart(task: DownloadTask) {
                //下载任务开始  根据需求提示
            }

            override fun infoReady(task: DownloadTask, info: BreakpointInfo, fromBreakpoint: Boolean, model: Listener4SpeedAssistExtend.Listener4SpeedModel) {
                totalLength = info.totalLength
                var apkPath = info.file?.absolutePath
                PreferencesUtil.saveStringByPremanent("apkpath${ToolBox.getString(R.string.app_name)}", apkPath)
//                LogUtil.e("apk--inforead--:$apkPath")
            }

            override fun connectStart(task: DownloadTask, blockIndex: Int, requestHeaderFields: MutableMap<String, MutableList<String>>) {
            }

            override fun connectEnd(task: DownloadTask, blockIndex: Int, responseCode: Int, responseHeaderFields: MutableMap<String, MutableList<String>>) {
            }

            override fun progressBlock(task: DownloadTask, blockIndex: Int, currentBlockOffset: Long, blockSpeed: SpeedCalculator) {
            }

            override fun progress(task: DownloadTask, currentOffset: Long, taskSpeed: SpeedCalculator) {
//                LogUtil.e("currentOffset:$currentOffset,totalLength:$totalLength---${(currentOffset.toFloat() / totalLength.toFloat())}")
                if (System.currentTimeMillis() - time > 1000) {
                    time = System.currentTimeMillis()
                    updateProgressBar(currentOffset, totalLength)
                }
            }

            override fun blockEnd(task: DownloadTask, blockIndex: Int, info: BlockInfo?, blockSpeed: SpeedCalculator) {
            }

            override fun taskEnd(task: DownloadTask, cause: EndCause, realCause: Exception?, taskSpeed: SpeedCalculator) {
//                LogUtil.e("taskEnd---cause:$cause---realCause:$realCause")
                if (cause == EndCause.COMPLETED) {
                    if (type == 2) {
                        progressBarTypeTwo?.let {
                            it.progress =  100
                        }
                        progressTitleTypeTwo?.text = "下载完成"
                    }
                    if (type == 1) {
//                        (mContext as Activity).runOnUiThread {
                            mCBuilder?.setContentTitle(ToolBox.getString(R.string.app_name))// 设置通知中心的标题
                            mCBuilder?.setContentText(ToolBox.getString(R.string.download_downs))
                            mCBuilder?.setProgress(100, 100, false)
                            mNotificationManager?.notify(NOTIFICATION_DOWNLOAD_CHANNEL_ID, 7, mCBuilder?.build())
//                        }
                    }

//                    LogUtil.e("task.file?.length():${task.file?.length()}---totallength:${task.file?.length()}")
                    if (task.file?.length() == totalLength){
                        installApk(task.file)
                    }
                }
                if (cause == EndCause.ERROR) {
                    //错误后再次重启下载
                    if (repeatTime < 20 && !handCancle) {
                        repeatTime++
                        task?.cancel()
                        Handler().postDelayed({
                            LogUtil.ee("重试------$repeatTime")
                            mContext ?: return@postDelayed
                            downloadApk()
                        }, 3000)
                    }

                }

            }
        })

    }


    /**
     * 获取文件夹路劲
     */
    fun getParentFile(): File {
        var apkDir: File? = null

        if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.P){
            if (Environment.getExternalStorageState() == Environment.MEDIA_MOUNTED) {
                // 获得存储卡的路径
                apkDir = Environment.getExternalStorageDirectory()
            } else {// 没有存储卡，则保存在内存里
                apkDir = mContext!!.getFilesDir()
            }
        }else{
            apkDir = mContext!!.getExternalFilesDir(Environment.DIRECTORY_PICTURES)
        }


        return apkDir
    }

    /**
     * 更新进度条   强制更新的
     */
    fun updateProgressBar(offset: Long, total: Long) {
        if (total == 0L) {
            return
        }
        var percent = offset.toFloat() / total.toFloat();
        if (type == 2) {
            progressBarTypeTwo?.let {
                it.progress = (percent * 100).toInt()
                progressTitleTypeTwo?.text = "正在下载:(${(percent * 100).toInt()}%)"
            }
            return
        }
        if (type == 1) {
//            (mContext as Activity).runOnUiThread {
                mCBuilder?.setProgress(100, (percent * 100).toInt(), false)
                mCBuilder?.setContentText("${(percent * 100).toInt()}%")
                mNotificationManager?.notify(NOTIFICATION_DOWNLOAD_CHANNEL_ID, 7, mCBuilder?.build())
//            }
            return
        }
    }

    /**
     * 取消下载任务
     */
    fun cancleTask() {
        task?.cancel()
    }

    /**
     * 安装APK文件
     */
    fun installApk(apkfile: File?) {
        if (mContext == null) {
            LogUtil.e("appupdate", "mContext为空")
            return
        }
        if (apkfile == null) {
            LogUtil.e("appupdate", "下载好的apkfile为空")
            return
        }
        //通知栏提示下载完成
//            val intentNotify = Intent(Intent.ACTION_VIEW)
//            val uri = CommonUtils.getUriWithPermission(mContext, apkfile, intentNotify)
//            intentNotify.setDataAndType(uri, "application/vnd.android.package-archive")
//            mPendingIntent = PendingIntent.getActivity(mContext, 0, intentNotify, 0)
//            mCBuilder?.setContentTitle(ToolBox.getString(R.string.app_name))// 设置通知中心的标题
//            mCBuilder?.setContentText(ToolBox.getString(R.string.download_downs))
//            mCBuilder?.setProgress(100, 100, false)
//            mCBuilder?.setContentIntent(mPendingIntent)
//            mNotificationManager?.notify(NOTIFICATION_DOWNLOAD_CHANNEL_ID, 7, mCBuilder?.build())

        // 通过Intent安装APK文件
        val apkPath = apkfile.absoluteFile.absolutePath
        val intent = Intent(Intent.ACTION_VIEW)
        // 由于没有在Activity环境下启动Activity,设置下面的标签
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {//判读版本是否在7.0以上
            //参数1 上下文, 参数2 Provider主机地址 和配置文件中保持一致   参数3  共享的文件
            val apkUri = FileProvider.getUriForFile(mContext!!, BuildConfig.APPLICATION_ID, apkfile)
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
            intent.setDataAndType(apkUri, "application/vnd.android.package-archive")
        } else {
            intent.setDataAndType(Uri.fromFile(apkfile),
                    "application/vnd.android.package-archive")
        }

        mContext?.startActivity(intent)
//        LogUtil.e("apk--安装--:$apkPath")
        // 保存下载的安装包信息(安装成功后，删除此apk——在MainA中提示)
        PreferencesUtil.saveStringByPremanent("apkpath${ToolBox.getString(R.string.app_name)}", apkPath)
    }


    /**
     * 创建通知栏提示
     */
    fun createNotification() {
        mContext ?: return
        mNotificationManager = mContext!!.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager?

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            mCBuilder = NotificationCompat.Builder(mContext!!.applicationContext, NOTIFICATION_DOWNLOAD_CHANNEL_ID)
            createNotificationChannel(NOTIFICATION_DOWNLOAD_CHANNEL_ID, NOTIFICATION_DOWNLOAD_CHANNEL_NAME, NotificationManager.IMPORTANCE_DEFAULT)
        } else {
            mCBuilder = NotificationCompat.Builder(mContext!!.applicationContext)
        }

        mCBuilder?.setSmallIcon(R.mipmap.ic_launcher)// 设置顶部状态栏的小图标
        mCBuilder?.setTicker(ToolBox.getString(R.string.downloadtip))// 在顶部状态栏中的提示信息
        mCBuilder?.setContentTitle(ToolBox.getString(R.string.app_name) + ToolBox.getString(R.string.downloading))// 设置通知中心的标题
        mCBuilder?.setContentText("0%")// 设置通知中心中的内容
        mCBuilder?.setWhen(System.currentTimeMillis())
        mCBuilder?.setProgress(100, 0, false)
        mNotification = mCBuilder?.build()
        mNotification?.flags = Notification.FLAG_ONGOING_EVENT
        mNotificationManager?.notify(NOTIFICATION_DOWNLOAD_CHANNEL_ID, 7, mNotification)//更新通知栏
    }

    @TargetApi(Build.VERSION_CODES.O)
    private fun createNotificationChannel(channelId: String, channelName: String, importance: Int) {
        val channel = NotificationChannel(channelId, channelName, importance)
        channel.enableVibration(false);
        channel.enableLights(true);
        channel.setSound(null, null);
        mNotificationManager?.createNotificationChannel(channel)
    }


}