package com.baojia.mebike.widget

import android.animation.ValueAnimator
import android.content.Context
import android.util.AttributeSet
import android.widget.TextView
import com.baojia.mebike.R
import com.baojia.mebike.util.ToolBox
import java.text.DecimalFormat
//数字增长textview
class RiseNumberTextView :TextView,IRaiseNumber{
    private val STOPPED = 0
    private val RUNNING = 1
    private var mPlayingState = STOPPED
    private var toNumber: Float = 0.toFloat()
    private var fromNumber: Float = 0.toFloat()
    private var duration: Long = 1500
    private var numberType = 2
    private val fnum = DecimalFormat("##0.00")
    private var mEndListener: EndListener? = null

    constructor(context: Context) :super(context)
    constructor(context: Context, attr: AttributeSet?) :super(context,attr)

    constructor(context: Context, attr: AttributeSet, defStyle: Int) :super(context,attr,defStyle)

    fun isRunning(): Boolean {
        //返回当前运行状态
        return mPlayingState == RUNNING
    }

    private fun runFloat() {
        val valueAnimator = ValueAnimator.ofFloat(fromNumber, toNumber)
        valueAnimator.duration = duration
        valueAnimator
                .addUpdateListener { valueAnimator ->
                    text = "￥"+fnum.format(java.lang.Float.parseFloat(valueAnimator
                            .animatedValue.toString()).toDouble())
                    if (valueAnimator.animatedFraction >= 1) {
                        //大于等于1时认为动画运行结束
                        mPlayingState = STOPPED
                        mEndListener?.onEndFinish()
                    }
                }
        valueAnimator.start()
    }

    private fun runInt() {
        val valueAnimator = ValueAnimator.ofInt(fromNumber.toInt(),
                toNumber.toInt())
        valueAnimator.duration = duration
        valueAnimator
                .addUpdateListener { valueAnimator ->
                    text = valueAnimator.animatedValue.toString()
                    if (valueAnimator.animatedFraction >= 1) {
                        //置标志位
                        mPlayingState = STOPPED
                        mEndListener?.onEndFinish()
                    }
                }
        valueAnimator.start()
    }

    override fun start() {
        if (!isRunning()) {
            mPlayingState = RUNNING
            if (numberType == 1)
                runInt()
            else
                runFloat()
        }
    }

    override fun setFloat(fromNum: Float, toNum: Float) {
        toNumber = toNum
        numberType = 2
        fromNumber = fromNum
    }

    override fun setInteger(fromNum: Int, toNum: Int) {
        toNumber = toNum.toFloat()
        numberType = 1
        fromNumber = fromNum.toFloat()
    }

    override fun setDuration(duration: Long) {
        this.duration = duration
    }

    override fun setOnEndListener(callback: EndListener) {
        mEndListener = callback
    }

    interface EndListener {
        fun onEndFinish()
    }

}


interface IRaiseNumber {
    fun start()
    fun setFloat(fromNum: Float, toNum: Float)
    fun setInteger(fromNum: Int, toNum: Int)
    fun setDuration(duration: Long)
    fun setOnEndListener(callback: RiseNumberTextView.EndListener)
}